# Sample UDev data for various MD RAID devices:
# - member_boot: data for the member disk or partition after booting the system
# - member_assemble: data for the member disk or partition after re-assembling stopped array using
#                    'mdadm --assemble --scan' (yes, this is different from member_boot)
# - raid_device: data for the RAID array device
# - raid_partition: data for partition on the array
#
# We have data for different combinations of member "types", MD metadata versions and named v unnamed
# RAID devices.
# The data were gathered on Fedora 32.


class RaidOnDisk1():
    member_name = "sda"
    raid_name = "127"
    raid_node = "md127"
    metadata_version = "1.2"

    member_boot = {'DEVLINKS': '/dev/disk/by-path/pci-0000:00:11.0-scsi-0:0:0:0 /dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-0',
                   'DEVNAME': '/dev/sda',
                   'DEVPATH': '/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:0/block/sda',
                   'DEVTYPE': 'disk',
                   'ID_BUS': 'scsi',
                   'ID_FS_LABEL': 'localhost.localdomain:127',
                   'ID_FS_LABEL_ENC': 'localhost.localdomain:127',
                   'ID_FS_TYPE': 'linux_raid_member',
                   'ID_FS_USAGE': 'raid',
                   'ID_FS_UUID': '54956eb2-6983-8759-e2ad-4c40acc92e4b',
                   'ID_FS_UUID_ENC': '54956eb2-6983-8759-e2ad-4c40acc92e4b',
                   'ID_FS_UUID_SUB': '64f96f0b-e97c-9157-d393-1fe457f3dd59',
                   'ID_FS_UUID_SUB_ENC': '64f96f0b-e97c-9157-d393-1fe457f3dd59',
                   'ID_FS_VERSION': '1.2',
                   'ID_MODEL': 'QEMU_HARDDISK',
                   'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                   'ID_PATH': 'pci-0000:00:11.0-scsi-0:0:0:0',
                   'ID_PATH_TAG': 'pci-0000_00_11_0-scsi-0_0_0_0',
                   'ID_REVISION': '2.5+',
                   'ID_SCSI': '1',
                   'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-0',
                   'ID_SERIAL_SHORT': 'drive-scsi0-0-0-0',
                   'ID_TYPE': 'disk',
                   'ID_VENDOR': 'QEMU',
                   'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                   'MAJOR': '8',
                   'MD_DEVICE': 'md127',
                   'MD_DEVNAME': '127',
                   'MD_FOREIGN': 'no',
                   'MD_STARTED': 'unsafe',
                   'MINOR': '0',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdadm-last-resort@md127.timer',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '5529231',
                   'SYS_NAME': 'sda',
                   'SYS_PATH': '/sys/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:0/block/sda'}

    member_assemble = {'DEVLINKS': '/dev/disk/by-path/pci-0000:00:11.0-scsi-0:0:0:0 /dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-0',
                       'DEVNAME': '/dev/sda',
                       'DEVPATH': '/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:0/block/sda',
                       'DEVTYPE': 'disk',
                       'ID_BUS': 'scsi',
                       'ID_FS_LABEL': 'localhost.localdomain:127',
                       'ID_FS_LABEL_ENC': 'localhost.localdomain:127',
                       'ID_FS_TYPE': 'linux_raid_member',
                       'ID_FS_USAGE': 'raid',
                       'ID_FS_UUID': '54956eb2-6983-8759-e2ad-4c40acc92e4b',
                       'ID_FS_UUID_ENC': '54956eb2-6983-8759-e2ad-4c40acc92e4b',
                       'ID_FS_UUID_SUB': '64f96f0b-e97c-9157-d393-1fe457f3dd59',
                       'ID_FS_UUID_SUB_ENC': '64f96f0b-e97c-9157-d393-1fe457f3dd59',
                       'ID_FS_VERSION': '1.2',
                       'ID_MODEL': 'QEMU_HARDDISK',
                       'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                       'ID_PATH': 'pci-0000:00:11.0-scsi-0:0:0:0',
                       'ID_PATH_TAG': 'pci-0000_00_11_0-scsi-0_0_0_0',
                       'ID_REVISION': '2.5+',
                       'ID_SCSI': '1',
                       'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-0',
                       'ID_SERIAL_SHORT': 'drive-scsi0-0-0-0',
                       'ID_TYPE': 'disk',
                       'ID_VENDOR': 'QEMU',
                       'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                       'MAJOR': '8',
                       'MINOR': '0',
                       'SUBSYSTEM': 'block',
                       'TAGS': ':systemd:',
                       'USEC_INITIALIZED': '5529231',
                       'SYS_NAME': 'sda',
                       'SYS_PATH': '/sys/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:0/block/sda'}

    raid_device = {'DEVLINKS': '/dev/disk/by-id/md-name-localhost.localdomain:127 /dev/disk/by-id/md-uuid-54956eb2:69838759:e2ad4c40:acc92e4b /dev/md/127',
                   'DEVNAME': '/dev/md127',
                   'DEVPATH': '/devices/virtual/block/md127',
                   'DEVTYPE': 'disk',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': '4eec0361',
                   'MAJOR': '9',
                   'MD_DEVICES': '2',
                   'MD_DEVICE_ev_sda_DEV': '/dev/sda',
                   'MD_DEVICE_ev_sda_ROLE': '0',
                   'MD_DEVICE_ev_sdb_DEV': '/dev/sdb',
                   'MD_DEVICE_ev_sdb_ROLE': '1',
                   'MD_DEVNAME': '127',
                   'MD_LEVEL': 'raid1',
                   'MD_METADATA': '1.2',
                   'MD_NAME': 'localhost.localdomain:127',
                   'MD_UUID': '54956eb2:69838759:e2ad4c40:acc92e4b',
                   'MINOR': '127',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdmonitor.service',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '603606045',
                   'SYS_NAME': 'md127',
                   'SYS_PATH': '/sys/devices/virtual/block/md127'}

    raid_partition = {'DEVLINKS': '/dev/md/127p1 /dev/disk/by-id/md-uuid-54956eb2:69838759:e2ad4c40:acc92e4b-part1 /dev/disk/by-id/md-name-localhost.localdomain:127-part1',
                      'DEVNAME': '/dev/md127p1',
                      'DEVPATH': '/devices/virtual/block/md127/md127p1',
                      'DEVTYPE': 'partition',
                      'ID_PART_ENTRY_DISK': '9:127',
                      'ID_PART_ENTRY_NUMBER': '1',
                      'ID_PART_ENTRY_OFFSET': '2048',
                      'ID_PART_ENTRY_SCHEME': 'dos',
                      'ID_PART_ENTRY_SIZE': '2091008',
                      'ID_PART_ENTRY_TYPE': '0x83',
                      'ID_PART_ENTRY_UUID': '4eec0361-01',
                      'MAJOR': '259',
                      'MD_DEVICES': '2',
                      'MD_DEVICE_ev_sda_DEV': '/dev/sda',
                      'MD_DEVICE_ev_sda_ROLE': '0',
                      'MD_DEVICE_ev_sdb_DEV': '/dev/sdb',
                      'MD_DEVICE_ev_sdb_ROLE': '1',
                      'MD_DEVNAME': '127',
                      'MD_LEVEL': 'raid1',
                      'MD_METADATA': '1.2',
                      'MD_NAME': 'localhost.localdomain:127',
                      'MD_UUID': '54956eb2:69838759:e2ad4c40:acc92e4b',
                      'MINOR': '2',
                      'PARTN': '1',
                      'SUBSYSTEM': 'block',
                      'SYSTEMD_WANTS': 'mdmonitor.service',
                      'TAGS': ':systemd:',
                      'USEC_INITIALIZED': '603714783',
                      'SYS_NAME': 'md127p1',
                      'SYS_PATH': '/sys/devices/virtual/block/md127/md127p1'}


class RaidOnDisk2():
    member_name = "sdc"
    raid_name = "name"
    raid_node = "md127"
    metadata_version = "1.2"

    member_boot = {'DEVLINKS': '/dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-4 /dev/disk/by-path/pci-0000:00:11.0-scsi-0:0:0:4',
                   'DEVNAME': '/dev/sdc',
                   'DEVPATH': '/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:4/block/sdc',
                   'DEVTYPE': 'disk',
                   'ID_BUS': 'scsi',
                   'ID_FS_LABEL': 'localhost.localdomain:name',
                   'ID_FS_LABEL_ENC': 'localhost.localdomain:name',
                   'ID_FS_TYPE': 'linux_raid_member',
                   'ID_FS_USAGE': 'raid',
                   'ID_FS_UUID': '143d480c-12c3-909f-5476-98a9f94a1c4f',
                   'ID_FS_UUID_ENC': '143d480c-12c3-909f-5476-98a9f94a1c4f',
                   'ID_FS_UUID_SUB': '121f2b71-3634-4183-dc9c-08bfceda765c',
                   'ID_FS_UUID_SUB_ENC': '121f2b71-3634-4183-dc9c-08bfceda765c',
                   'ID_FS_VERSION': '1.2',
                   'ID_MODEL': 'QEMU_HARDDISK',
                   'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                   'ID_PATH': 'pci-0000:00:11.0-scsi-0:0:0:4',
                   'ID_PATH_TAG': 'pci-0000_00_11_0-scsi-0_0_0_4',
                   'ID_REVISION': '2.5+',
                   'ID_SCSI': '1',
                   'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-4',
                   'ID_SERIAL_SHORT': 'drive-scsi0-0-0-4',
                   'ID_TYPE': 'disk',
                   'ID_VENDOR': 'QEMU',
                   'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                   'MAJOR': '8',
                   'MD_DEVICE': 'md127',
                   'MD_DEVNAME': 'name',
                   'MD_FOREIGN': 'no',
                   'MD_STARTED': 'yes',
                   'MINOR': '32',
                   'SUBSYSTEM': 'block',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '6109555',
                   'SYS_NAME': 'sdc',
                   'SYS_PATH': '/sys/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:4/block/sdc'}

    member_assemble = {'DEVLINKS': '/dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-4 /dev/disk/by-path/pci-0000:00:11.0-scsi-0:0:0:4',
                       'DEVNAME': '/dev/sdc',
                       'DEVPATH': '/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:4/block/sdc',
                       'DEVTYPE': 'disk',
                       'ID_BUS': 'scsi',
                       'ID_FS_LABEL': 'localhost.localdomain:name',
                       'ID_FS_LABEL_ENC': 'localhost.localdomain:name',
                       'ID_FS_TYPE': 'linux_raid_member',
                       'ID_FS_USAGE': 'raid',
                       'ID_FS_UUID': '143d480c-12c3-909f-5476-98a9f94a1c4f',
                       'ID_FS_UUID_ENC': '143d480c-12c3-909f-5476-98a9f94a1c4f',
                       'ID_FS_UUID_SUB': '121f2b71-3634-4183-dc9c-08bfceda765c',
                       'ID_FS_UUID_SUB_ENC': '121f2b71-3634-4183-dc9c-08bfceda765c',
                       'ID_FS_VERSION': '1.2',
                       'ID_MODEL': 'QEMU_HARDDISK',
                       'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                       'ID_PATH': 'pci-0000:00:11.0-scsi-0:0:0:4',
                       'ID_PATH_TAG': 'pci-0000_00_11_0-scsi-0_0_0_4',
                       'ID_REVISION': '2.5+',
                       'ID_SCSI': '1',
                       'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-4',
                       'ID_SERIAL_SHORT': 'drive-scsi0-0-0-4',
                       'ID_TYPE': 'disk',
                       'ID_VENDOR': 'QEMU',
                       'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                       'MAJOR': '8',
                       'MINOR': '32',
                       'SUBSYSTEM': 'block',
                       'TAGS': ':systemd:',
                       'USEC_INITIALIZED': '6109555',
                       'SYS_NAME': 'sdc',
                       'SYS_PATH': '/sys/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:4/block/sdc'}

    raid_device = {'DEVLINKS': '/dev/disk/by-id/md-name-localhost.localdomain:name /dev/disk/by-id/md-uuid-143d480c:12c3909f:547698a9:f94a1c4f /dev/md/name',
                   'DEVNAME': '/dev/md127',
                   'DEVPATH': '/devices/virtual/block/md127',
                   'DEVTYPE': 'disk',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': '19e9cb5b',
                   'MAJOR': '9',
                   'MD_DEVICES': '2',
                   'MD_DEVICE_ev_sdc_DEV': '/dev/sdc',
                   'MD_DEVICE_ev_sdc_ROLE': '0',
                   'MD_DEVICE_ev_sdd_DEV': '/dev/sdd',
                   'MD_DEVICE_ev_sdd_ROLE': '1',
                   'MD_DEVNAME': 'name',
                   'MD_LEVEL': 'raid1',
                   'MD_METADATA': '1.2',
                   'MD_NAME': 'localhost.localdomain:name',
                   'MD_UUID': '143d480c:12c3909f:547698a9:f94a1c4f',
                   'MINOR': '127',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdmonitor.service',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '5844744',
                   'SYS_NAME': 'md127',
                   'SYS_PATH': '/sys/devices/virtual/block/md127'}

    raid_partition = {'DEVLINKS': '/dev/disk/by-id/md-uuid-143d480c:12c3909f:547698a9:f94a1c4f-part1 /dev/disk/by-id/md-name-localhost.localdomain:name-part1 /dev/md/name1',
                      'DEVNAME': '/dev/md127p1',
                      'DEVPATH': '/devices/virtual/block/md127/md127p1',
                      'DEVTYPE': 'partition',
                      'ID_PART_ENTRY_DISK': '9:127',
                      'ID_PART_ENTRY_NUMBER': '1',
                      'ID_PART_ENTRY_OFFSET': '2048',
                      'ID_PART_ENTRY_SCHEME': 'dos',
                      'ID_PART_ENTRY_SIZE': '2091008',
                      'ID_PART_ENTRY_TYPE': '0x83',
                      'ID_PART_ENTRY_UUID': '19e9cb5b-01',
                      'ID_PART_TABLE_TYPE': 'dos',
                      'ID_PART_TABLE_UUID': 'ec985633',
                      'MAJOR': '259',
                      'MD_DEVICES': '2',
                      'MD_DEVICE_ev_sdc_DEV': '/dev/sdc',
                      'MD_DEVICE_ev_sdc_ROLE': '0',
                      'MD_DEVICE_ev_sdd_DEV': '/dev/sdd',
                      'MD_DEVICE_ev_sdd_ROLE': '1',
                      'MD_DEVNAME': 'name',
                      'MD_LEVEL': 'raid1',
                      'MD_METADATA': '1.2',
                      'MD_NAME': 'localhost.localdomain:name',
                      'MD_UUID': '143d480c:12c3909f:547698a9:f94a1c4f',
                      'MINOR': '1',
                      'PARTN': '1',
                      'SUBSYSTEM': 'block',
                      'SYSTEMD_WANTS': 'mdmonitor.service',
                      'TAGS': ':systemd:',
                      'USEC_INITIALIZED': '5928255',
                      'SYS_NAME': 'md127p1',
                      'SYS_PATH': '/sys/devices/virtual/block/md127/md127p1'}


class RaidOnDisk3():
    member_name = "sde"
    raid_name = "125"
    raid_node = "md125"
    metadata_version = "0.9"

    member_boot = {'DEVLINKS': '/dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-1 /dev/disk/by-path/pci-0000:00:11.0-scsi-0:0:0:1',
                   'DEVNAME': '/dev/sde',
                   'DEVPATH': '/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:1/block/sde',
                   'DEVTYPE': 'disk',
                   'ID_BUS': 'scsi',
                   'ID_FS_TYPE': 'linux_raid_member',
                   'ID_FS_USAGE': 'raid',
                   'ID_FS_UUID': 'c4ef60f5-e374-5f70-bfe7-8010bc810f04',
                   'ID_FS_UUID_ENC': 'c4ef60f5-e374-5f70-bfe7-8010bc810f04',
                   'ID_FS_VERSION': '0.90.0',
                   'ID_MODEL': 'QEMU_HARDDISK',
                   'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                   'ID_PATH': 'pci-0000:00:11.0-scsi-0:0:0:1',
                   'ID_PATH_TAG': 'pci-0000_00_11_0-scsi-0_0_0_1',
                   'ID_REVISION': '2.5+',
                   'ID_SCSI': '1',
                   'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-1',
                   'ID_SERIAL_SHORT': 'drive-scsi0-0-0-1',
                   'ID_TYPE': 'disk',
                   'ID_VENDOR': 'QEMU',
                   'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                   'MAJOR': '8',
                   'MD_DEVICE': 'md125',
                   'MD_DEVNAME': '125',
                   'MD_FOREIGN': 'no',
                   'MD_STARTED': 'unsafe',
                   'MINOR': '64',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdadm-last-resort@md125.timer',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '5538551',
                   'SYS_NAME': 'sde',
                   'SYS_PATH': '/sys/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:1/block/sde'}

    member_assemble = {'DEVLINKS': '/dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-1 /dev/disk/by-path/pci-0000:00:11.0-scsi-0:0:0:1',
                       'DEVNAME': '/dev/sde',
                       'DEVPATH': '/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:1/block/sde',
                       'DEVTYPE': 'disk',
                       'ID_BUS': 'scsi',
                       'ID_FS_TYPE': 'linux_raid_member',
                       'ID_FS_USAGE': 'raid',
                       'ID_FS_UUID': 'c4ef60f5-e374-5f70-bfe7-8010bc810f04',
                       'ID_FS_UUID_ENC': 'c4ef60f5-e374-5f70-bfe7-8010bc810f04',
                       'ID_FS_VERSION': '0.90.0',
                       'ID_MODEL': 'QEMU_HARDDISK',
                       'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                       'ID_PATH': 'pci-0000:00:11.0-scsi-0:0:0:1',
                       'ID_PATH_TAG': 'pci-0000_00_11_0-scsi-0_0_0_1',
                       'ID_REVISION': '2.5+',
                       'ID_SCSI': '1',
                       'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi0-0-0-1',
                       'ID_SERIAL_SHORT': 'drive-scsi0-0-0-1',
                       'ID_TYPE': 'disk',
                       'ID_VENDOR': 'QEMU',
                       'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                       'MAJOR': '8',
                       'MINOR': '64',
                       'SUBSYSTEM': 'block',
                       'TAGS': ':systemd:',
                       'USEC_INITIALIZED': '5538551',
                       'SYS_NAME': 'sde',
                       'SYS_PATH': '/sys/devices/pci0000:00/0000:00:11.0/virtio5/host8/target8:0:0/8:0:0:1/block/sde'}

    raid_device = {'DEVLINKS': '/dev/md/125 /dev/disk/by-id/md-uuid-c4ef60f5:e3745f70:bfe78010:bc810f04',
                   'DEVNAME': '/dev/md125',
                   'DEVPATH': '/devices/virtual/block/md125',
                   'DEVTYPE': 'disk',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': 'e74877cd',
                   'MAJOR': '9',
                   'MD_DEVICES': '2',
                   'MD_DEVICE_ev_sde_DEV': '/dev/sde',
                   'MD_DEVICE_ev_sde_ROLE': '0',
                   'MD_DEVICE_ev_sdf_DEV': '/dev/sdf',
                   'MD_DEVICE_ev_sdf_ROLE': '1',
                   'MD_DEVNAME': '125',
                   'MD_LEVEL': 'raid1',
                   'MD_METADATA': '0.90',
                   'MD_UUID': 'c4ef60f5:e3745f70:bfe78010:bc810f04',
                   'MINOR': '125',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdmonitor.service',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '5786380',
                   'SYS_NAME': 'md125',
                   'SYS_PATH': '/sys/devices/virtual/block/md125'}

    raid_partition = {'DEVLINKS': '/dev/md/125p1 /dev/disk/by-id/md-uuid-c4ef60f5:e3745f70:bfe78010:bc810f04-part1',
                      'DEVNAME': '/dev/md125p1',
                      'DEVPATH': '/devices/virtual/block/md125/md125p1',
                      'DEVTYPE': 'partition',
                      'ID_PART_ENTRY_DISK': '9:125',
                      'ID_PART_ENTRY_NUMBER': '1',
                      'ID_PART_ENTRY_OFFSET': '2048',
                      'ID_PART_ENTRY_SCHEME': 'dos',
                      'ID_PART_ENTRY_SIZE': '2094976',
                      'ID_PART_ENTRY_TYPE': '0x83',
                      'ID_PART_ENTRY_UUID': 'e74877cd-01',
                      'MAJOR': '259',
                      'MD_DEVICES': '2',
                      'MD_DEVICE_ev_sde_DEV': '/dev/sde',
                      'MD_DEVICE_ev_sde_ROLE': '0',
                      'MD_DEVICE_ev_sdf_DEV': '/dev/sdf',
                      'MD_DEVICE_ev_sdf_ROLE': '1',
                      'MD_DEVNAME': '125',
                      'MD_LEVEL': 'raid1',
                      'MD_METADATA': '0.90',
                      'MD_UUID': 'c4ef60f5:e3745f70:bfe78010:bc810f04',
                      'MINOR': '3',
                      'PARTN': '1',
                      'SUBSYSTEM': 'block',
                      'SYSTEMD_WANTS': 'mdmonitor.service',
                      'TAGS': ':systemd:',
                      'USEC_INITIALIZED': '8808457',
                      'SYS_NAME': 'md125p1',
                      'SYS_PATH': '/sys/devices/virtual/block/md125/md125p1'}


class RaidOnPartition1():
    member_name = "sdh3"
    raid_name = "122"
    raid_node = "md122"
    metadata_version = "1.2"

    member_boot = {'DEVLINKS': '/dev/disk/by-path/pci-0000:00:07.0-scsi-0:0:2:0-part3 /dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi1-0-2-part3 /dev/disk/by-partuuid/73eb11a9-03',
                   'DEVNAME': '/dev/sdh3',
                   'DEVPATH': '/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh3',
                   'DEVTYPE': 'partition',
                   'ID_BUS': 'scsi',
                   'ID_FS_LABEL': 'localhost.localdomain:122',
                   'ID_FS_LABEL_ENC': 'localhost.localdomain:122',
                   'ID_FS_TYPE': 'linux_raid_member',
                   'ID_FS_USAGE': 'raid',
                   'ID_FS_UUID': '0628d995-eb60-ebd1-a767-51730b16f212',
                   'ID_FS_UUID_ENC': '0628d995-eb60-ebd1-a767-51730b16f212',
                   'ID_FS_UUID_SUB': 'b301779b-f759-ad7d-5324-b38d4b6d944d',
                   'ID_FS_UUID_SUB_ENC': 'b301779b-f759-ad7d-5324-b38d4b6d944d',
                   'ID_FS_VERSION': '1.2',
                   'ID_MODEL': 'QEMU_HARDDISK',
                   'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                   'ID_PART_ENTRY_DISK': '8:112',
                   'ID_PART_ENTRY_NUMBER': '3',
                   'ID_PART_ENTRY_OFFSET': '411648',
                   'ID_PART_ENTRY_SCHEME': 'dos',
                   'ID_PART_ENTRY_SIZE': '204800',
                   'ID_PART_ENTRY_TYPE': '0x83',
                   'ID_PART_ENTRY_UUID': '73eb11a9-03',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': '73eb11a9',
                   'ID_PATH': 'pci-0000:00:07.0-scsi-0:0:2:0',
                   'ID_PATH_TAG': 'pci-0000_00_07_0-scsi-0_0_2_0',
                   'ID_REVISION': '2.5+',
                   'ID_SCSI': '1',
                   'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi1-0-2',
                   'ID_SERIAL_SHORT': 'drive-scsi1-0-2',
                   'ID_TYPE': 'disk',
                   'ID_VENDOR': 'QEMU',
                   'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                   'MAJOR': '8',
                   'MD_DEVICE': 'md122',
                   'MD_DEVNAME': '122',
                   'MD_FOREIGN': 'no',
                   'MD_STARTED': 'yes',
                   'MINOR': '115',
                   'PARTN': '3',
                   'SUBSYSTEM': 'block',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '8920462',
                   'SYS_NAME': 'sdh3',
                   'SYS_PATH': '/sys/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh3'}

    member_assemble = {'DEVLINKS': '/dev/disk/by-path/pci-0000:00:07.0-scsi-0:0:2:0-part3 /dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi1-0-2-part3 /dev/disk/by-partuuid/73eb11a9-03',
                       'DEVNAME': '/dev/sdh3',
                       'DEVPATH': '/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh3',
                       'DEVTYPE': 'partition',
                       'ID_BUS': 'scsi',
                       'ID_FS_LABEL': 'localhost.localdomain:122',
                       'ID_FS_LABEL_ENC': 'localhost.localdomain:122',
                       'ID_FS_TYPE': 'linux_raid_member',
                       'ID_FS_USAGE': 'raid',
                       'ID_FS_UUID': '0628d995-eb60-ebd1-a767-51730b16f212',
                       'ID_FS_UUID_ENC': '0628d995-eb60-ebd1-a767-51730b16f212',
                       'ID_FS_UUID_SUB': 'b301779b-f759-ad7d-5324-b38d4b6d944d',
                       'ID_FS_UUID_SUB_ENC': 'b301779b-f759-ad7d-5324-b38d4b6d944d',
                       'ID_FS_VERSION': '1.2',
                       'ID_MODEL': 'QEMU_HARDDISK',
                       'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                       'ID_PART_ENTRY_DISK': '8:112',
                       'ID_PART_ENTRY_NUMBER': '3',
                       'ID_PART_ENTRY_OFFSET': '411648',
                       'ID_PART_ENTRY_SCHEME': 'dos',
                       'ID_PART_ENTRY_SIZE': '204800',
                       'ID_PART_ENTRY_TYPE': '0x83',
                       'ID_PART_ENTRY_UUID': '73eb11a9-03',
                       'ID_PART_TABLE_TYPE': 'dos',
                       'ID_PART_TABLE_UUID': '73eb11a9',
                       'ID_PATH': 'pci-0000:00:07.0-scsi-0:0:2:0',
                       'ID_PATH_TAG': 'pci-0000_00_07_0-scsi-0_0_2_0',
                       'ID_REVISION': '2.5+',
                       'ID_SCSI': '1',
                       'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi1-0-2',
                       'ID_SERIAL_SHORT': 'drive-scsi1-0-2',
                       'ID_TYPE': 'disk',
                       'ID_VENDOR': 'QEMU',
                       'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                       'MAJOR': '8',
                       'MINOR': '115',
                       'PARTN': '3',
                       'SUBSYSTEM': 'block',
                       'TAGS': ':systemd:',
                       'USEC_INITIALIZED': '8920462',
                       'SYS_NAME': 'sdh3',
                       'SYS_PATH': '/sys/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh3'}

    raid_device = {'DEVLINKS': '/dev/disk/by-id/md-uuid-0628d995:eb60ebd1:a7675173:0b16f212 /dev/disk/by-id/md-name-localhost.localdomain:122 /dev/md/122',
                   'DEVNAME': '/dev/md122',
                   'DEVPATH': '/devices/virtual/block/md122',
                   'DEVTYPE': 'disk',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': '6dc80b3b',
                   'MAJOR': '9',
                   'MD_DEVICES': '2',
                   'MD_DEVICE_ev_sdh3_DEV': '/dev/sdh3',
                   'MD_DEVICE_ev_sdh3_ROLE': '0',
                   'MD_DEVICE_ev_sdh5_DEV': '/dev/sdh5',
                   'MD_DEVICE_ev_sdh5_ROLE': '1',
                   'MD_DEVNAME': '122',
                   'MD_LEVEL': 'raid1',
                   'MD_METADATA': '1.2',
                   'MD_NAME': 'localhost.localdomain:122',
                   'MD_UUID': '0628d995:eb60ebd1:a7675173:0b16f212',
                   'MINOR': '122',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdmonitor.service',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '8770105',
                   'SYS_NAME': 'md122',
                   'SYS_PATH': '/sys/devices/virtual/block/md122'}

    raid_partition = {'DEVLINKS': '/dev/disk/by-id/md-uuid-0628d995:eb60ebd1:a7675173:0b16f212-part1 /dev/disk/by-id/md-name-localhost.localdomain:122-part1 /dev/md/122p1',
                      'DEVNAME': '/dev/md122p1',
                      'DEVPATH': '/devices/virtual/block/md122/md122p1',
                      'DEVTYPE': 'partition',
                      'ID_PART_ENTRY_DISK': '9:122',
                      'ID_PART_ENTRY_NUMBER': '1',
                      'ID_PART_ENTRY_OFFSET': '2048',
                      'ID_PART_ENTRY_SCHEME': 'dos',
                      'ID_PART_ENTRY_SIZE': '200704',
                      'ID_PART_ENTRY_TYPE': '0x83',
                      'ID_PART_ENTRY_UUID': '6dc80b3b-01',
                      'MAJOR': '259',
                      'MD_DEVICES': '2',
                      'MD_DEVICE_ev_sdh3_DEV': '/dev/sdh3',
                      'MD_DEVICE_ev_sdh3_ROLE': '0',
                      'MD_DEVICE_ev_sdh5_DEV': '/dev/sdh5',
                      'MD_DEVICE_ev_sdh5_ROLE': '1',
                      'MD_DEVNAME': '122',
                      'MD_LEVEL': 'raid1',
                      'MD_METADATA': '1.2',
                      'MD_NAME': 'localhost.localdomain:122',
                      'MD_UUID': '0628d995:eb60ebd1:a7675173:0b16f212',
                      'MINOR': '6',
                      'PARTN': '1',
                      'SUBSYSTEM': 'block',
                      'SYSTEMD_WANTS': 'mdmonitor.service',
                      'TAGS': ':systemd:',
                      'USEC_INITIALIZED': '9003885',
                      'SYS_NAME': 'md122p1',
                      'SYS_PATH': '/sys/devices/virtual/block/md122/md122p1'}


class RaidOnPartition2():
    member_name = "sdh1"
    raid_name = "123"
    raid_node = "md123"
    metadata_version = "0.9"

    member_boot = {'DEVLINKS': '/dev/disk/by-path/pci-0000:00:07.0-scsi-0:0:2:0-part1 /dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi1-0-2-part1 /dev/disk/by-partuuid/73eb11a9-01',
                   'DEVNAME': '/dev/sdh1',
                   'DEVPATH': '/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh1',
                   'DEVTYPE': 'partition',
                   'ID_BUS': 'scsi',
                   'ID_FS_TYPE': 'linux_raid_member',
                   'ID_FS_USAGE': 'raid',
                   'ID_FS_UUID': '335b35e0-f1af-8e86-bfe7-8010bc810f04',
                   'ID_FS_UUID_ENC': '335b35e0-f1af-8e86-bfe7-8010bc810f04',
                   'ID_FS_VERSION': '0.90.0',
                   'ID_MODEL': 'QEMU_HARDDISK',
                   'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                   'ID_PART_ENTRY_DISK': '8:112',
                   'ID_PART_ENTRY_NUMBER': '1',
                   'ID_PART_ENTRY_OFFSET': '2048',
                   'ID_PART_ENTRY_SCHEME': 'dos',
                   'ID_PART_ENTRY_SIZE': '204800',
                   'ID_PART_ENTRY_TYPE': '0x83',
                   'ID_PART_ENTRY_UUID': '73eb11a9-01',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': '73eb11a9',
                   'ID_PATH': 'pci-0000:00:07.0-scsi-0:0:2:0',
                   'ID_PATH_TAG': 'pci-0000_00_07_0-scsi-0_0_2_0',
                   'ID_REVISION': '2.5+',
                   'ID_SCSI': '1',
                   'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi1-0-2',
                   'ID_SERIAL_SHORT': 'drive-scsi1-0-2',
                   'ID_TYPE': 'disk',
                   'ID_VENDOR': 'QEMU',
                   'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                   'MAJOR': '8',
                   'MD_DEVICE': 'md123',
                   'MD_DEVNAME': '123',
                   'MD_FOREIGN': 'no',
                   'MD_STARTED': 'unsafe',
                   'MINOR': '113',
                   'PARTN': '1',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdadm-last-resort@md123.timer',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '8778733',
                   'SYS_NAME': 'sdh1',
                   'SYS_PATH': '/sys/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh1'}

    member_assemble = {'DEVLINKS': '/dev/disk/by-path/pci-0000:00:07.0-scsi-0:0:2:0-part1 /dev/disk/by-id/scsi-0QEMU_QEMU_HARDDISK_drive-scsi1-0-2-part1 /dev/disk/by-partuuid/73eb11a9-01',
                       'DEVNAME': '/dev/sdh1',
                       'DEVPATH': '/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh1',
                       'DEVTYPE': 'partition',
                       'ID_BUS': 'scsi',
                       'ID_FS_TYPE': 'linux_raid_member',
                       'ID_FS_USAGE': 'raid',
                       'ID_FS_UUID': '335b35e0-f1af-8e86-bfe7-8010bc810f04',
                       'ID_FS_UUID_ENC': '335b35e0-f1af-8e86-bfe7-8010bc810f04',
                       'ID_FS_VERSION': '0.90.0',
                       'ID_MODEL': 'QEMU_HARDDISK',
                       'ID_MODEL_ENC': 'QEMU\\x20HARDDISK\\x20\\x20\\x20',
                       'ID_PART_ENTRY_DISK': '8:112',
                       'ID_PART_ENTRY_NUMBER': '1',
                       'ID_PART_ENTRY_OFFSET': '2048',
                       'ID_PART_ENTRY_SCHEME': 'dos',
                       'ID_PART_ENTRY_SIZE': '204800',
                       'ID_PART_ENTRY_TYPE': '0x83',
                       'ID_PART_ENTRY_UUID': '73eb11a9-01',
                       'ID_PART_TABLE_TYPE': 'dos',
                       'ID_PART_TABLE_UUID': '73eb11a9',
                       'ID_PATH': 'pci-0000:00:07.0-scsi-0:0:2:0',
                       'ID_PATH_TAG': 'pci-0000_00_07_0-scsi-0_0_2_0',
                       'ID_REVISION': '2.5+',
                       'ID_SCSI': '1',
                       'ID_SERIAL': '0QEMU_QEMU_HARDDISK_drive-scsi1-0-2',
                       'ID_SERIAL_SHORT': 'drive-scsi1-0-2',
                       'ID_TYPE': 'disk',
                       'ID_VENDOR': 'QEMU',
                       'ID_VENDOR_ENC': 'QEMU\\x20\\x20\\x20\\x20',
                       'MAJOR': '8',
                       'MINOR': '113',
                       'PARTN': '1',
                       'SUBSYSTEM': 'block',
                       'TAGS': ':systemd:',
                       'UDISKS_MD_MEMBER_DEVICES': '2',
                       'UDISKS_MD_MEMBER_EVENTS': '18',
                       'UDISKS_MD_MEMBER_LEVEL': 'raid1',
                       'UDISKS_MD_MEMBER_UPDATE_TIME': '1597143914',
                       'UDISKS_MD_MEMBER_UUID': '335b35e0:f1af8e86:bfe78010:bc810f04',
                       'USEC_INITIALIZED': '8778733',
                       'SYS_NAME': 'sdh1',
                       'SYS_PATH': '/sys/devices/pci0000:00/0000:00:07.0/host9/target9:0:2/9:0:2:0/block/sdh/sdh1'}

    raid_device = {'DEVLINKS': '/dev/md/123 /dev/disk/by-id/md-uuid-335b35e0:f1af8e86:bfe78010:bc810f04',
                   'DEVNAME': '/dev/md123',
                   'DEVPATH': '/devices/virtual/block/md123',
                   'DEVTYPE': 'disk',
                   'ID_PART_TABLE_TYPE': 'dos',
                   'ID_PART_TABLE_UUID': '653f84c8',
                   'MAJOR': '9',
                   'MD_DEVICES': '2',
                   'MD_DEVICE_ev_sdh1_DEV': '/dev/sdh1',
                   'MD_DEVICE_ev_sdh1_ROLE': '0',
                   'MD_DEVICE_ev_sdh2_DEV': '/dev/sdh2',
                   'MD_DEVICE_ev_sdh2_ROLE': '1',
                   'MD_DEVNAME': '123',
                   'MD_LEVEL': 'raid1',
                   'MD_METADATA': '0.90',
                   'MD_UUID': '335b35e0:f1af8e86:bfe78010:bc810f04',
                   'MINOR': '123',
                   'SUBSYSTEM': 'block',
                   'SYSTEMD_WANTS': 'mdmonitor.service',
                   'TAGS': ':systemd:',
                   'USEC_INITIALIZED': '8760382',
                   'SYS_NAME': 'md123',
                   'SYS_PATH': '/sys/devices/virtual/block/md123'}

    raid_partition = {'DEVLINKS': '/dev/disk/by-id/md-uuid-335b35e0:f1af8e86:bfe78010:bc810f04-part1 /dev/md/123p1',
                      'DEVNAME': '/dev/md123p1',
                      'DEVPATH': '/devices/virtual/block/md123/md123p1',
                      'DEVTYPE': 'partition',
                      'ID_PART_ENTRY_DISK': '9:123',
                      'ID_PART_ENTRY_NUMBER': '1',
                      'ID_PART_ENTRY_OFFSET': '2048',
                      'ID_PART_ENTRY_SCHEME': 'dos',
                      'ID_PART_ENTRY_SIZE': '202624',
                      'ID_PART_ENTRY_TYPE': '0x83',
                      'ID_PART_ENTRY_UUID': '653f84c8-01',
                      'MAJOR': '259',
                      'MD_DEVICES': '2',
                      'MD_DEVICE_ev_sdh1_DEV': '/dev/sdh1',
                      'MD_DEVICE_ev_sdh1_ROLE': '0',
                      'MD_DEVICE_ev_sdh2_DEV': '/dev/sdh2',
                      'MD_DEVICE_ev_sdh2_ROLE': '1',
                      'MD_DEVNAME': '123',
                      'MD_LEVEL': 'raid1',
                      'MD_METADATA': '0.90',
                      'MD_UUID': '335b35e0:f1af8e86:bfe78010:bc810f04',
                      'MINOR': '5',
                      'PARTN': '1',
                      'SUBSYSTEM': 'block',
                      'SYSTEMD_WANTS': 'mdmonitor.service',
                      'TAGS': ':systemd:',
                      'USEC_INITIALIZED': '8952876',
                      'SYS_NAME': 'md123p1',
                      'SYS_PATH': '/sys/devices/virtual/block/md123/md123p1'}



# Check translations of pango markup
#
# This will look for translatable strings that appear to contain markup and
# check that the markup in the translation matches.
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Shea <dshea@redhat.com>

try:
    import polib
except ImportError:
    print("You need to install the python-polib package to read translations")
    raise

from pocketlint.pangocheck import is_markup, markup_match
import xml.etree.ElementTree as ET


def test_markup(pofile):
    po = polib.pofile(pofile)

    for entry in po.translated_entries():
        if is_markup(entry.msgid):
            # If this is a plural, check each of the plural translations
            if entry.msgid_plural:
                xlations = entry.msgstr_plural
            else:
                xlations = {None: entry.msgstr}

            for plural_id, msgstr in xlations.items():
                # Check if the markup is valid at all
                try:
                    # pylint: disable=unescaped-markup
                    ET.fromstring('<markup>%s</markup>' % msgstr)
                except ET.ParseError as e:
                    if entry.msgid_plural:
                        raise AssertionError(
                            "Invalid markup translation for {} translation of msgid {}\n{}".
                            format(plural_id, entry.msgid, msgstr)) from e
                    else:
                        raise AssertionError(
                            "Invalid markup translation for msgid {}\n{}".
                            format(entry.msgid, msgstr)) from e

                # Check if the markup has the same number and kind of tags
                if not markup_match(entry.msgid, msgstr):
                    if entry.msgid_plural:
                        raise AssertionError(
                            "Markup does not match for {} translation of msgid {}\n{}".
                            format(plural_id, entry.msgid, msgstr))
                    else:
                        raise AssertionError(
                            "Markup does not match for msgid {}\n{}".
                            format(entry.msgid, msgstr))



# vim:set fileencoding=utf-8

import unittest

from blivet.devices import NetworkStorageDevice
from blivet.devices import StorageDevice

from blivet.formats import get_format


class FakeNetDev(StorageDevice, NetworkStorageDevice):
    _type = "fakenetdev"


class NetDevMountOptionTestCase(unittest.TestCase):

    def test_net_dev_setting(self):
        """ Verify netdev mount option setting after format assignment. """
        netdev = FakeNetDev("net1")
        dev = StorageDevice("dev1", fmt=get_format("ext4"))
        self.assertFalse("_netdev" in dev.format.options.split(","))

        dev.parents.append(netdev)
        dev.format = get_format("ext4")
        self.assertTrue("_netdev" in dev.format.options.split(","))

    def test_net_dev_update(self):
        """ Verify netdev mount option setting after device creation. """
        netdev = FakeNetDev("net1")
        dev = StorageDevice("dev1", fmt=get_format("ext4"))
        self.assertFalse("_netdev" in dev.format.options.split(","))

        dev.parents.append(netdev)

        # these create methods shouldn't write anything to disk
        netdev.create()
        dev.create()

        self.assertTrue("_netdev" in dev.format.options.split(","))

    def test_net_dev_update_remove(self):
        """ Verify netdev mount option is removed after removing the netdev parent. """
        netdev = FakeNetDev("net1")
        dev = StorageDevice("dev1", parents=[netdev], fmt=get_format("ext4"))
        self.assertTrue("_netdev" in dev.format.options.split(","))

        dev.parents.remove(netdev)

        # these create methods shouldn't write anything to disk
        netdev.create()
        dev.create()

        self.assertFalse("_netdev" in dev.format.options.split(","))

    def test_net_device_manual(self):
        """ Verify netdev mount option is not removed if explicitly set by the user. """
        dev = StorageDevice("dev1", fmt=get_format("ext4", mountopts="_netdev"))
        self.assertTrue("_netdev" in dev.format.options.split(","))

        # these create methods shouldn't write anything to disk
        dev.create()

        self.assertTrue("_netdev" in dev.format.options.split(","))



# fsmkfs.py
# Filesystem formatting classes.
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Anne Mulhern <amulhern@redhat.com>

import abc
import shlex

from six import add_metaclass

from ..errors import FSError, FSWriteLabelError, FSWriteUUIDError
from .. import util

from . import availability
from . import fstask
from . import task


@add_metaclass(abc.ABCMeta)
class FSMkfsTask(fstask.FSTask):

    can_label = abc.abstractproperty(doc="whether this task labels")
    can_set_uuid = abc.abstractproperty(doc="whether this task can set UUID")
    can_nodiscard = abc.abstractproperty(doc="whether this task can set nodiscard option")


@add_metaclass(abc.ABCMeta)
class FSMkfs(task.BasicApplication, FSMkfsTask):

    """An abstract class that represents filesystem creation actions. """
    description = "mkfs"

    label_option = abc.abstractproperty(
        doc="Option for setting a filesystem label.")

    nodiscard_option = abc.abstractproperty(
        doc="Option for setting nodiscrad option for mkfs.")

    args = abc.abstractproperty(doc="options for creating filesystem")

    @abc.abstractmethod
    def get_uuid_args(self, uuid):
        """Return a list of arguments for setting a filesystem UUID.

           :param uuid: the UUID to set
           :type uuid: str
           :rtype: list of str
        """
        raise NotImplementedError

    # IMPLEMENTATION methods

    @property
    def can_label(self):
        """ Whether this task can label the filesystem.

            :returns: True if this task can label the filesystem
            :rtype: bool
        """
        return self.label_option is not None

    @property
    def can_set_uuid(self):
        """Whether this task can set the UUID of a filesystem.

           :returns: True if UUID can be set
           :rtype: bool
        """
        return self.get_uuid_args is not None

    @property
    def can_nodiscard(self):
        """Whether this task can set nodiscard option for a filesystem.

           :returns: True if nodiscard can be set
           :rtype: bool
        """
        return self.nodiscard_option is not None

    @property
    def _label_options(self):
        """ Any labeling options that a particular filesystem may use.

            :returns: labeling options
            :rtype: list of str
        """
        # Do not know how to set label while formatting.
        if self.label_option is None:
            return []

        # No label to set
        if self.fs.label is None:
            return []

        if self.fs.label_format_ok(self.fs.label):
            return [self.label_option, self.fs.label]
        else:
            raise FSWriteLabelError("Choosing not to apply label (%s) during creation of filesystem %s. Label format is unacceptable for this filesystem." % (self.fs.label, self.fs.type))

    @property
    def _nodiscard_option(self):
        """ Any nodiscard options that a particular filesystem may use.

            :returns: nodiscard options
            :rtype: list of str
        """
        # Do not know how to set nodiscard while formatting.
        if self.nodiscard_option is None:
            return []

        # nodiscard option not requested
        if not self.fs._mkfs_nodiscard:
            return []

        return self.nodiscard_option

    @property
    def _uuid_options(self):
        """Any UUID options that a particular filesystem may use.

           :returns: UUID options
           :rtype: list of str
           :raises: FSWriteUUIDError
        """
        if self.get_uuid_args is None or self.fs.uuid is None:
            return []

        if self.fs.uuid_format_ok(self.fs.uuid):
            return self.get_uuid_args(self.fs.uuid)
        else:
            raise FSWriteUUIDError("Choosing not to apply UUID (%s) during"
                                   " creation of filesystem %s. UUID format"
                                   " is unacceptable for this filesystem."
                                   % (self.fs.uuid, self.fs.type))

    def _format_options(self, options=None, label=False, set_uuid=False, nodiscard=False):
        """Get a list of format options to be used when creating the
           filesystem.

           :param options: any special options
           :type options: list of str or None
           :param bool label: if True, label if possible, default is False
           :param bool set_uuid: whether set UUID if possible, default is False
        """
        options = options or []

        if not isinstance(options, list):
            raise FSError("options parameter must be a list.")

        label_options = self._label_options if label else []
        uuid_options = self._uuid_options if set_uuid else []
        nodiscard_option = self._nodiscard_option if nodiscard else []
        create_options = shlex.split(self.fs.create_options or "")
        return (options + self.args + label_options + uuid_options +
                nodiscard_option + create_options + [self.fs.device])

    def _mkfs_command(self, options, label, set_uuid, nodiscard):
        """Return the command to make the filesystem.

           :param options: any special options
           :type options: list of str or None
           :param label: whether to set a label
           :type label: bool
           :param set_uuid: whether to set an UUID
           :type set_uuid: bool
           :param nodiscard: whether to run mkfs with nodiscard option
           :type nodiscard: bool
           :returns: the mkfs command
           :rtype: list of str
        """
        return [str(self.ext)] + self._format_options(options, label, set_uuid, nodiscard)

    def do_task(self, options=None, label=False, set_uuid=False, nodiscard=False):
        """Create the format on the device and label if possible and desired.

           :param options: any special options, may be None
           :type options: list of str or NoneType
           :param bool label: whether to label while creating, default is False
           :param bool set_uuid: whether to set an UUID while creating, default
                                 is False
        """
        # pylint: disable=arguments-differ
        error_msgs = self.availability_errors
        if error_msgs:
            raise FSError("\n".join(error_msgs))

        options = options or []
        cmd = self._mkfs_command(options, label, set_uuid, nodiscard)
        try:
            ret = util.run_program(cmd)
        except OSError as e:
            raise FSError(e)

        if ret:
            raise FSError("format failed: %s" % ret)


class BTRFSMkfs(FSMkfs):
    ext = availability.MKFS_BTRFS_APP
    label_option = None
    nodiscard_option = ["--nodiscard"]

    def get_uuid_args(self, uuid):
        return ["-U", uuid]

    @property
    def args(self):
        return []


class Ext2FSMkfs(FSMkfs):
    ext = availability.MKE2FS_APP
    label_option = "-L"
    nodiscard_option = ["-E", "nodiscard"]

    _opts = []

    def get_uuid_args(self, uuid):
        return ["-U", uuid]

    @property
    def args(self):
        return self._opts + (["-T", self.fs.fsprofile] if self.fs.fsprofile else [])


class Ext3FSMkfs(Ext2FSMkfs):
    _opts = ["-t", "ext3"]


class Ext4FSMkfs(Ext3FSMkfs):
    _opts = ["-t", "ext4"]


class FATFSMkfs(FSMkfs):
    ext = availability.MKDOSFS_APP
    label_option = "-n"
    nodiscard_option = None

    def get_uuid_args(self, uuid):
        return ["-i", uuid.replace('-', '')]

    @property
    def args(self):
        return []


class GFS2Mkfs(FSMkfs):
    ext = availability.MKFS_GFS2_APP
    label_option = None
    nodiscard_option = None
    get_uuid_args = None

    @property
    def args(self):
        return ["-j", "1", "-p", "lock_nolock", "-O"]


class HFSMkfs(FSMkfs):
    ext = availability.HFORMAT_APP
    label_option = "-l"
    nodiscard_option = None
    get_uuid_args = None

    @property
    def args(self):
        return []


class HFSPlusMkfs(FSMkfs):
    ext = availability.MKFS_HFSPLUS_APP
    label_option = "-v"
    nodiscard_option = None
    get_uuid_args = None

    @property
    def args(self):
        return []


class JFSMkfs(FSMkfs):
    ext = availability.MKFS_JFS_APP
    label_option = "-L"
    nodiscard_option = None
    get_uuid_args = None

    @property
    def args(self):
        return ["-q"]


class NTFSMkfs(FSMkfs):
    ext = availability.MKNTFS_APP
    label_option = "-L"
    nodiscard_option = None
    get_uuid_args = None

    @property
    def args(self):
        # -F (force) to allow creating the format on disks, -f (fast) to skip zeroing the device
        return ["-F", "-f"]


class ReiserFSMkfs(FSMkfs):
    ext = availability.MKREISERFS_APP
    label_option = "-l"
    nodiscard_option = None

    def get_uuid_args(self, uuid):
        return ["-u", uuid]

    @property
    def args(self):
        return ["-f", "-f"]


class XFSMkfs(FSMkfs):
    ext = availability.MKFS_XFS_APP
    label_option = "-L"
    nodiscard_option = ["-K"]

    def get_uuid_args(self, uuid):
        return ["-m", "uuid=" + uuid]

    @property
    def args(self):
        return ["-f"]


class F2FSMkfs(FSMkfs):
    ext = availability.MKFS_F2FS_APP
    label_option = "-l"
    nodiscard_option = ["-t", "nodiscard"]
    get_uuid_args = None

    @property
    def args(self):
        # Enable the extended node bitmap, this means that we can create more
        # files and directories without running out of inodes, even if the
        # available space for metadata is limited.
        return ["-i"]


class UnimplementedFSMkfs(task.UnimplementedTask, FSMkfsTask):

    @property
    def can_label(self):
        return False

    @property
    def can_set_uuid(self):
        return False

    @property
    def can_nodiscard(self):
        return False



# devices/lib.py
#
# Copyright (C) 2009-2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#
from enum import Enum
import os

from .. import errors
from .. import udev
from ..size import Size

LINUX_SECTOR_SIZE = Size(512)


class Tags(str, Enum):
    """Tags that describe various classes of disk."""
    local = 'local'
    nvdimm = 'nvdimm'
    nvme = 'nvme'
    remote = 'remote'
    removable = 'removable'
    ssd = 'ssd'
    usb = 'usb'


def _collect_device_major_data():
    by_major = {}
    by_device = {}
    for line in open("/proc/devices").readlines():
        try:
            (major, device) = line.split()
        except ValueError:
            continue
        try:
            by_major[int(major)] = device
            if device not in by_device:
                by_device[device] = []

            by_device[device].append(int(major))
        except ValueError:
            continue
    return (by_major, by_device)


_devices_by_major, _majors_by_device = _collect_device_major_data()


def get_majors_by_device_type(device_type):
    """ Return a list of major numbers for the given device type.

        :param str device_type: device type string (eg: 'device-mapper', 'md')
        :rtype: list of int

        .. note::

            Type strings are taken directly from /proc/devices.

    """
    return _majors_by_device.get(device_type, [])


def get_device_type_by_major(major):
    """ Return a device-type string for the given major number.

        :param int major: major number
        :rtype: str

        .. note::

            Type strings are taken directly from /proc/devices.

    """
    return _devices_by_major.get(major)


def device_path_to_name(device_path):
    """ Return a name based on the given path to a device node.

        :param device_path: the path to a device node
        :type device_path: str
        :returns: the name
        :rtype: str
    """
    if not device_path:
        return None

    if device_path.startswith("/dev/"):
        name = device_path[5:]
    else:
        name = device_path

    if name.startswith("mapper/"):
        name = name[7:]

    if name.startswith("md/"):
        name = name[3:]

    if name.startswith("/"):
        name = os.path.basename(name)

    return name


def device_name_to_disk_by_path(device_name=None):
    """ Return a /dev/disk/by-path/ symlink path for the given device name.

        :param device_name: the device name
        :type device_name: str
        :returns: the full path to a /dev/disk/by-path/ symlink, or None
        :rtype: str or NoneType
    """
    if not device_name:
        return ""

    ret = None
    for dev in udev.get_devices():
        if udev.device_get_name(dev) == device_name:
            ret = udev.device_get_by_path(dev)
            break

    if ret:
        return ret
    raise errors.DeviceNotFoundError(device_name)


class ParentList(object):

    """ A list with auditing and side-effects for additions and removals.

        The class provides an ordered list with guaranteed unique members and
        optional functions to run before adding or removing a member. It
        provides a subset of the functionality provided by :class:`list`,
        making it easy to ensure that changes pass through the check functions.

        The following operations are implemented:

        .. code::

            ml.append(x)
            ml.remove(x)
            iter(ml)
            len(ml)
            x in ml
            x = ml[i]   # not ml[i] = x
    """

    def __init__(self, items=None, appendfunc=None, removefunc=None):
        """
            :keyword items: initial contents
            :type items: any iterable
            :keyword appendfunc: a function to call before adding an item
            :type appendfunc: callable
            :keyword removefunc: a function to call before removing an item
            :type removefunc: callable

            appendfunc and removefunc should take the item to be added or
            removed and perform any checks or other processing. The appropriate
            function will be called immediately before adding or removing the
            item. The function should raise an exception if the addition/removal
            should not take place. :class:`~.ParentList` instance is not passed
            to the function. While this is not optimal for general-purpose use,
            it is ideal for the intended use as part of :class:`~.Device`. The
            functions themselves should not modify the :class:`~.ParentList`.
        """
        self.items = list()
        if items:
            self.items.extend(items)

        self.appendfunc = appendfunc or (lambda i: True)
        """ a function to call before adding an item """

        self.removefunc = removefunc or (lambda i: True)
        """ a function to call before removing an item """

    def __iter__(self):
        return iter(self.items)

    def __contains__(self, y):
        return y in self.items

    def __getitem__(self, i):
        return self.items[i]

    def __len__(self):
        return len(self.items)

    def __str__(self):
        return str(self.items)

    def __repr__(self):
        return repr(self.items)

    def append(self, y):
        """ Add an item to the list after running a callback. """
        if y in self.items:
            raise ValueError("item is already in the list")

        self.appendfunc(y)
        self.items.append(y)

    def remove(self, y):
        """ Remove an item from the list after running a callback. """
        if y not in self.items:
            raise ValueError("item is not in the list")

        self.removefunc(y)
        self.items.remove(y)




import dbus

bus = dbus.SystemBus()

# This adds a signal match so that the client gets signals sent by Blivet1's
# ObjectManager. These signals are used to notify clients of changes to the
# managed objects (for blivet, this will be devices, formats, and actions).
bus.add_match_string("type='signal',sender='com.redhat.Blivet1',path_namespace='/com/redhat/Blivet1'")

blivet = bus.get_object('com.redhat.Blivet1', '/com/redhat/Blivet1/Blivet')
blivet.Reset()

object_manager = bus.get_object('com.redhat.Blivet1', '/com/redhat/Blivet1')
objects = object_manager.GetManagedObjects()
for object_path in blivet.ListDevices():
    device = objects[object_path]['com.redhat.Blivet1.Device']
    fmt = objects[device['Format']]['com.redhat.Blivet1.Format']
    print(device['Name'], device['Type'], device['Size'], fmt['Type'])



import os

import blivet
from blivet.size import Size
from blivet.util import set_up_logging, create_sparse_tempfile

set_up_logging()
b = blivet.Blivet()   # create an instance of Blivet (don't add system devices)

# create a disk image file on which to create new devices
disk1_file = create_sparse_tempfile("disk1", Size("100GiB"))
b.disk_images["disk1"] = disk1_file

b.reset()

try:
    disk1 = b.devicetree.get_device_by_name("disk1")

    b.initialize_disk(disk1)

    pv = b.new_partition(size=Size("50GiB"), fmt_type="lvmpv")
    b.create_device(pv)

    # allocate the partitions (decide where and on which disks they'll reside)
    blivet.partitioning.do_partitioning(b)

    vg = b.new_vg(parents=[pv])
    b.create_device(vg)

    # new 40 GiB thin pool
    pool = b.new_lv(thin_pool=True, size=Size("40 GiB"), parents=[vg])
    b.create_device(pool)

    # new 20 GiB thin lv
    thinlv = b.new_lv(thin_volume=True, size=Size("20 GiB"), parents=[pool],
                      fmt_type="ext4")
    b.create_device(thinlv)

    # new snapshot of the thin volume we just created
    snap = b.new_lv(name=thinlv.name + "_snapshot", parents=[pool], origin=thinlv,
                    seg_type="thin")
    b.create_device(snap)

    # write the new partitions to disk and format them as specified
    b.do_it()
    print(b.devicetree)
finally:
    b.devicetree.teardown_disk_images()
    os.unlink(disk1_file)



import glob
import os
import re
import shutil
import subprocess
import unittest

from contextlib import contextmanager

from .storagetestcase import create_sparse_tempfile


def read_file(filename, mode="r"):
    with open(filename, mode) as f:
        content = f.read()
    return content


@contextmanager
def udev_settle():
    try:
        yield
    finally:
        os.system("udevadm settle")


def _delete_backstore(name):
    status = subprocess.call(["targetcli", "/backstores/fileio/ delete %s" % name],
                             stdout=subprocess.DEVNULL)
    if status != 0:
        raise RuntimeError("Failed to delete the '%s' fileio backstore" % name)


def delete_iscsi_target(iqn, backstore=None):
    status = subprocess.call(["targetcli", "/iscsi delete %s" % iqn],
                             stdout=subprocess.DEVNULL)
    if status != 0:
        raise RuntimeError("Failed to delete the '%s' iscsi device" % iqn)

    if backstore is not None:
        _delete_backstore(backstore)


def create_iscsi_target(fpath, initiator_name=None):
    """
    Creates a new iSCSI target (using targetcli) on top of the
    :param:`fpath` backing file.

    :param str fpath: path of the backing file
    :returns: iSCSI IQN, backstore name
    :rtype: tuple of str

    """

    # "register" the backing file as a fileio backstore
    store_name = os.path.basename(fpath)
    status = subprocess.call(["targetcli", "/backstores/fileio/ create %s %s" % (store_name, fpath)], stdout=subprocess.DEVNULL)
    if status != 0:
        raise RuntimeError("Failed to register '%s' as a fileio backstore" % fpath)

    out = subprocess.check_output(["targetcli", "/backstores/fileio/%s info" % store_name])
    out = out.decode("utf-8")
    store_wwn = None
    for line in out.splitlines():
        if line.startswith("wwn: "):
            store_wwn = line[5:]
    if store_wwn is None:
        raise RuntimeError("Failed to determine '%s' backstore's wwn" % store_name)

    # create a new iscsi device
    out = subprocess.check_output(["targetcli", "/iscsi create"])
    out = out.decode("utf-8")
    match = re.match(r'Created target (.*).', out)
    if match:
        iqn = match.groups()[0]
    else:
        _delete_backstore(store_name)
        raise RuntimeError("Failed to create a new iscsi target")

    with udev_settle():
        status = subprocess.call(["targetcli", "/iscsi/%s/tpg1/luns create /backstores/fileio/%s" % (iqn, store_name)], stdout=subprocess.DEVNULL)
    if status != 0:
        delete_iscsi_target(iqn, store_name)
        raise RuntimeError("Failed to create a new LUN for '%s' using '%s'" % (iqn, store_name))

    if initiator_name:
        status = subprocess.call(["targetcli", "/iscsi/%s/tpg1/acls create %s" % (iqn, initiator_name)], stdout=subprocess.DEVNULL)
        if status != 0:
            delete_iscsi_target(iqn, store_name)
            raise RuntimeError("Failed to set ACLs for '%s'" % iqn)

    return iqn, store_name


@unittest.skipUnless(os.geteuid() == 0, "requires root privileges")
@unittest.skipUnless(os.environ.get("JENKINS_HOME"), "jenkins only test")
@unittest.skipUnless(shutil.which("iscsiadm"), "iscsiadm not available")
class ISCSITestCase(unittest.TestCase):

    _disk_size = 512 * 1024**2
    initiator = 'iqn.1994-05.com.redhat:iscsi-test'

    def setUp(self):
        self.addCleanup(self._clean_up)

        self._dev_file = None
        self.dev = None

        self._dev_file = create_sparse_tempfile("blivet_test", self._disk_size)
        try:
            self.dev, self.backstore = create_iscsi_target(self._dev_file, self.initiator)
        except RuntimeError as e:
            raise RuntimeError("Failed to setup targetcli device for testing: %s" % e)

    def _force_logout(self):
        subprocess.call(["iscsiadm", "--mode", "node", "--logout", "--name", self.dev], stdout=subprocess.DEVNULL)

    def _clean_up(self):
        self._force_logout()
        delete_iscsi_target(self.dev, self.backstore)
        os.unlink(self._dev_file)

    def test_discover_login(self):
        from blivet.iscsi import iscsi, has_iscsi

        if not has_iscsi() or not iscsi.available:
            self.skipTest("iSCSI not available, skipping")

        # initially set the initiator to the correct/allowed one
        iscsi.initiator = self.initiator
        nodes = iscsi.discover("127.0.0.1")
        self.assertTrue(nodes)

        if len(nodes) > 1:
            self.skipTest("Discovered more than one iSCSI target on localhost, skipping")

        self.assertEqual(nodes[0].address, "127.0.0.1")
        self.assertEqual(nodes[0].port, 3260)
        self.assertEqual(nodes[0].name, self.dev)

        # change the initiator name to a wrong one
        iscsi.initiator = self.initiator + "_1"
        self.assertEqual(iscsi.initiator, self.initiator + "_1")

        # check the change made it to /etc/iscsi/initiatorname.iscsi
        initiator_file = read_file("/etc/iscsi/initiatorname.iscsi").strip()
        self.assertEqual(initiator_file, "InitiatorName=%s" % self.initiator + "_1")

        # try to login (should fail)
        ret, err = iscsi.log_into_node(nodes[0])
        self.assertFalse(ret)
        self.assertIn("authorization failure", err)

        # change the initiator name back to the correct one
        iscsi.initiator = self.initiator
        self.assertEqual(iscsi.initiator, self.initiator)

        # check the change made it to /etc/iscsi/initiatorname.iscsi
        initiator_file = read_file("/etc/iscsi/initiatorname.iscsi").strip()
        self.assertEqual(initiator_file, "InitiatorName=%s" % self.initiator)

        # try to login (should work now)
        ret, err = iscsi.log_into_node(nodes[0])
        self.assertTrue(ret, "Login failed: %s" % err)

        # check the session for initiator name
        sessions = glob.glob("/sys/class/iscsi_session/*/")
        self.assertTrue(sessions)
        self.assertEqual(len(sessions), 1)
        initiator = read_file(sessions[0] + "initiatorname").strip()
        self.assertEqual(initiator, iscsi.initiator)



import os

import blivet
from blivet.size import Size
from blivet.util import set_up_logging, create_sparse_tempfile
from blivet.devices.lvm import LVMCacheRequest, LVMCacheType

set_up_logging()
b = blivet.Blivet()   # create an instance of Blivet (don't add system devices)

# create a disk image file on which to create new devices
disk1_file = create_sparse_tempfile("disk1", Size("100GiB"))
b.disk_images["disk1"] = disk1_file
disk2_file = create_sparse_tempfile("disk2", Size("100GiB"))
b.disk_images["disk2"] = disk2_file

b.reset()

try:
    disk1 = b.devicetree.get_device_by_name("disk1")
    disk2 = b.devicetree.get_device_by_name("disk2")

    b.initialize_disk(disk1)
    b.initialize_disk(disk2)

    pv = b.new_partition(size=Size("50GiB"), fmt_type="lvmpv", parents=[disk1])
    b.create_device(pv)
    pv2 = b.new_partition(size=Size("50GiB"), fmt_type="lvmpv", parents=[disk2])
    b.create_device(pv2)

    # allocate the partitions (decide where and on which disks they'll reside)
    blivet.partitioning.do_partitioning(b)

    vg = b.new_vg(parents=[pv, pv2])
    b.create_device(vg)

    # new lv cached by a standard LVM cache
    cache_spec = LVMCacheRequest(size=Size("1GiB"), pvs=[pv2])
    dev = b.new_lv(fmt_type="ext4", size=Size("2GiB"), parents=[vg], name="cached", cache_request=cache_spec)
    b.create_device(dev)

    # new lv cached by a LVM writecache
    cache_spec = LVMCacheRequest(size=Size("1GiB"), pvs=[pv2], cache_type=LVMCacheType.lvmwritecache)
    dev = b.new_lv(fmt_type="ext4", size=Size("2GiB"), parents=[vg], name="writecached", cache_request=cache_spec)
    b.create_device(dev)

    # allocate the growable lvs
    blivet.partitioning.grow_lvm(b)
    print(b.devicetree)

    # write the new partitions to disk and format them as specified
    b.do_it()
    print(b.devicetree)
    input("Check the state and hit ENTER to trigger cleanup")
finally:
    b.devicetree.teardown_disk_images()
    os.unlink(disk1_file)
    os.unlink(disk2_file)



# stratis_info.py
# Backend code for populating a DeviceTree.
#
# Copyright (C) 2020  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): Vojtech Trefny <vtrefny@redhat.com>
#

import os
import uuid

from collections import namedtuple

from .. import safe_dbus
from ..size import Size

import logging
log = logging.getLogger("blivet")


# XXX we can't import these from devicelibs.stratis, circular imports make python mad
STRATIS_SERVICE = "org.storage.stratis3"
STRATIS_PATH = "/org/storage/stratis3"
STRATIS_POOL_INTF = STRATIS_SERVICE + ".pool.r0"
STRATIS_FILESYSTEM_INTF = STRATIS_SERVICE + ".filesystem.r0"
STRATIS_BLOCKDEV_INTF = STRATIS_SERVICE + ".blockdev.r0"
STRATIS_MANAGER_INTF = STRATIS_SERVICE + ".Manager.r0"


StratisPoolInfo = namedtuple("StratisPoolInfo", ["name", "uuid", "physical_size", "physical_used", "object_path", "encrypted"])
StratisFilesystemInfo = namedtuple("StratisFilesystemInfo", ["name", "uuid", "used_size", "pool_name",
                                                             "pool_uuid", "object_path"])
StratisBlockdevInfo = namedtuple("StratisBlockdevInfo", ["path", "uuid", "pool_name", "pool_uuid", "object_path"])
StratisLockedPoolInfo = namedtuple("StratisLockedPoolInfo", ["uuid", "key_desc", "devices"])


class StratisInfo(object):
    """ Class to be used as a singleton.
        Maintains the Stratis devices info cache.
    """

    def __init__(self):
        self._info_cache = None

    def _get_pool_info(self, pool_path):
        try:
            properties = safe_dbus.get_properties_sync(STRATIS_SERVICE,
                                                       pool_path,
                                                       STRATIS_POOL_INTF)[0]
        except safe_dbus.DBusPropertyError as e:
            log.error("Error when getting DBus properties of '%s': %s",
                      pool_path, str(e))

        if not properties:
            log.error("Failed to get DBus properties of '%s'", pool_path)
            return None

        pool_size = properties.get("TotalPhysicalSize", 0)

        valid, pool_used = properties.get("TotalPhysicalUsed",
                                          (False, "TotalPhysicalUsed not available"))
        if not valid:
            log.warning("Failed to get Stratis pool physical used for %s: %s",
                        properties["Name"], pool_used)
            pool_used = 0

        return StratisPoolInfo(name=properties["Name"], uuid=properties["Uuid"],
                               physical_size=Size(pool_size), physical_used=Size(pool_used),
                               object_path=pool_path, encrypted=properties["Encrypted"])

    def _get_filesystem_info(self, filesystem_path):
        try:
            properties = safe_dbus.get_properties_sync(STRATIS_SERVICE,
                                                       filesystem_path,
                                                       STRATIS_FILESYSTEM_INTF)[0]
        except safe_dbus.DBusPropertyError as e:
            log.error("Error when getting DBus properties of '%s': %s",
                      filesystem_path, str(e))

        if not properties:
            log.error("Failed to get DBus properties of '%s'", filesystem_path)
            return None

        pool_info = self._get_pool_info(properties["Pool"])
        if not pool_info:
            return None

        valid, used_size = properties.get("Used",
                                          (False, "Used not available"))
        if not valid:
            log.warning("Failed to get Stratis filesystem used size for %s: %s",
                        properties["Name"], used_size)
            used_size = 0

        return StratisFilesystemInfo(name=properties["Name"], uuid=properties["Uuid"],
                                     used_size=Size(used_size),
                                     pool_name=pool_info.name, pool_uuid=pool_info.uuid,
                                     object_path=filesystem_path)

    def _get_blockdev_info(self, blockdev_path):
        try:
            properties = safe_dbus.get_properties_sync(STRATIS_SERVICE,
                                                       blockdev_path,
                                                       STRATIS_BLOCKDEV_INTF)[0]
        except safe_dbus.DBusPropertyError as e:
            log.error("Error when getting DBus properties of '%s': %s",
                      blockdev_path, str(e))

        if not properties:
            log.error("Failed to get DBus properties of '%s'", blockdev_path)
            return None

        blockdev_uuid = str(uuid.UUID(properties["Uuid"]))

        pool_path = properties["Pool"]
        if pool_path == "/":
            pool_name = ""
            return StratisBlockdevInfo(path=properties["Devnode"], uuid=blockdev_uuid,
                                       pool_name="", pool_uuid="", object_path=blockdev_path)
        else:
            pool_info = self._get_pool_info(properties["Pool"])
            if not pool_info:
                return None
            pool_name = pool_info.name

            return StratisBlockdevInfo(path=properties["Devnode"], uuid=blockdev_uuid,
                                       pool_name=pool_name, pool_uuid=pool_info.uuid,
                                       object_path=blockdev_path)

    def _get_locked_pools_info(self):
        locked_pools = []

        try:
            pools_info = safe_dbus.get_property_sync(STRATIS_SERVICE,
                                                     STRATIS_PATH,
                                                     STRATIS_MANAGER_INTF,
                                                     "LockedPools")[0]
        except safe_dbus.DBusCallError as e:
            log.error("Failed to get list of locked Stratis pools: %s", str(e))
            return locked_pools

        for pool_uuid in pools_info.keys():
            valid, (_err, description) = pools_info[pool_uuid]["key_description"]
            if not valid:
                log.info("Locked Stratis pool %s doesn't have a valid key description: %s", pool_uuid, description)
                description = None
            info = StratisLockedPoolInfo(uuid=pool_uuid,
                                         key_desc=description,
                                         devices=[d["devnode"] for d in pools_info[pool_uuid]["devs"]])
            locked_pools.append(info)

        return locked_pools

    def _get_stratis_info(self):
        self._info_cache = dict()
        self._info_cache["pools"] = dict()
        self._info_cache["blockdevs"] = dict()
        self._info_cache["filesystems"] = dict()
        self._info_cache["locked_pools"] = []

        try:
            ret = safe_dbus.check_object_available(STRATIS_SERVICE, STRATIS_PATH)
        except safe_dbus.DBusCallError:
            log.warning("Stratis DBus service is not running")
            return
        else:
            if not ret:
                log.warning("Stratis DBus service is not available")

        objects = safe_dbus.call_sync(STRATIS_SERVICE,
                                      STRATIS_PATH,
                                      "org.freedesktop.DBus.ObjectManager",
                                      "GetManagedObjects",
                                      None)[0]

        for path, interfaces in objects.items():
            if STRATIS_POOL_INTF in interfaces.keys():
                pool_info = self._get_pool_info(path)
                if pool_info:
                    self._info_cache["pools"][pool_info.uuid] = pool_info

            if STRATIS_FILESYSTEM_INTF in interfaces.keys():
                fs_info = self._get_filesystem_info(path)
                if fs_info:
                    self._info_cache["filesystems"][fs_info.uuid] = fs_info

            if STRATIS_BLOCKDEV_INTF in interfaces.keys():
                bd_info = self._get_blockdev_info(path)
                if bd_info:
                    self._info_cache["blockdevs"][bd_info.uuid] = bd_info

        self._info_cache["locked_pools"] = self._get_locked_pools_info()

    @property
    def pools(self):
        if self._info_cache is None:
            self._get_stratis_info()

        return self._info_cache["pools"]

    @property
    def filesystems(self):
        if self._info_cache is None:
            self._get_stratis_info()

        return self._info_cache["filesystems"]

    @property
    def blockdevs(self):
        if self._info_cache is None:
            self._get_stratis_info()

        return self._info_cache["blockdevs"]

    @property
    def locked_pools(self):
        if self._info_cache is None:
            self._get_stratis_info()

        return self._info_cache["locked_pools"]

    def drop_cache(self):
        self._info_cache = None

    def get_pool_info(self, pool_name):
        for pool in self.pools.values():
            if pool.name == pool_name:
                return pool

    def get_filesystem_info(self, pool_name, fs_name):
        for fs in self.filesystems.values():
            if fs.pool_name == pool_name and fs.name == fs_name:
                return fs

    def get_blockdev_info(self, bd_path):
        for bd in self.blockdevs.values():
            if bd.path == bd_path or bd.path == os.path.realpath(bd_path):
                return bd


stratis_info = StratisInfo()



# populator/helpers/formatpopulator.py
# Base classes for type-specific helpers for populating a DeviceTree.
#
# Copyright (C) 2009-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

from ...callbacks import callbacks
from ... import formats
from ... import udev
from ...errors import FSError

from .populatorhelper import PopulatorHelper

import logging
log = logging.getLogger("blivet")


class FormatPopulator(PopulatorHelper):
    priority = 0
    _type_specifier = None

    @classmethod
    def match(cls, data, device):  # pylint: disable=arguments-differ,unused-argument
        """ Return True if this helper is appropriate for the given device.

            :param :class:`pyudev.Device` data: udev data describing a device
            :param device: device instance corresponding to the udev data
            :type device: :class:`~.devices.StorageDevice`
            :returns: whether this class is appropriate for the specified device
            :rtype: bool
        """
        if device.type == "stratis filesystem":
            # XFS on stratis filesystem device handled in StratisXFSFormatPopulator
            return False

        if cls is FormatPopulator:
            ret = True
        else:
            format_class = formats.get_device_format_class(cls._type_specifier)
            fmt_types = []
            if format_class is not None:
                fmt_types = format_class._udev_types[:]
                if cls._type_specifier and cls._type_specifier not in fmt_types:
                    fmt_types.append(cls._type_specifier)

            ret = (udev.device_get_format(data) in fmt_types)

        return ret

    def _get_kwargs(self):
        """ Return a kwargs dict to pass to DeviceFormat constructor. """
        kwargs = {"uuid": udev.device_get_uuid(self.data),
                  "label": udev.device_get_label(self.data),
                  "device": self.device.path,
                  "serial": udev.device_get_serial(self.data),
                  "exists": True}
        return kwargs

    @property
    def type_spec(self):
        if self._type_specifier is not None:
            type_spec = self._type_specifier
        else:
            type_spec = udev.device_get_format(self.data) or None

        return type_spec

    def run(self):
        """ Create a format instance and associate it with the device instance. """
        kwargs = self._get_kwargs()
        type_spec = self.type_spec
        try:
            log.info("type detected on '%s' is '%s'", self.device.name, type_spec)
            self.device.format = formats.get_format(type_spec, **kwargs)
        except FSError:
            log.warning("type '%s' on '%s' invalid, assuming no format",
                        type_spec, self.device.name)
            self.device.format = formats.DeviceFormat(device=self.device.path, exists=True)

    def update(self):
        label = udev.device_get_label(self.data)
        if hasattr(self.device.format, "label") and self.device.format.label != label:
            old_label = self.device.format.label
            self.device.format.label = label
            callbacks.attribute_changed(device=self.device, fmt=self.device.format,
                                        attr="label", old=old_label, new=label)



try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

import unittest

from blivet.formats.luks import LUKS
from blivet.size import Size
from blivet.static_data import luks_data


class LUKSNodevTestCase(unittest.TestCase):
    def setUp(self):
        luks_data.pbkdf_args = None

    def test_create_discard_option(self):
        # flags.discard_new=False --> no discard
        fmt = LUKS(exists=False)
        self.assertEqual(fmt.options, None)

        fmt = LUKS(exists=True)
        self.assertEqual(fmt.options, None)

        # flags.discard_new=True --> discard if creating new
        with patch("blivet.flags.flags.discard_new", True):
            fmt = LUKS(exists=True)
            self.assertEqual(fmt.options, None)

            fmt = LUKS(exists=False)
            self.assertEqual(fmt.options, "discard")

            # do not add if already there
            fmt = LUKS(exists=False, options="discard")
            self.assertEqual(fmt.options, "discard")

            # add with comma after other option(s)
            fmt = LUKS(exists=False, options="blah")
            self.assertEqual(fmt.options, "blah,discard")

    def test_key_size(self):
        # default cipher is AES-XTS with 512b key
        fmt = LUKS()
        self.assertEqual(fmt.cipher, "aes-xts-plain64")
        self.assertEqual(fmt.key_size, 512)

        # setting cipher shouldn't change the key size
        fmt = LUKS(cipher="aes-xts-plain64")
        self.assertEqual(fmt.key_size, 512)

        # all XTS mode ciphers should default to 512
        fmt = LUKS(cipher="serpent-xts-plain64")
        self.assertEqual(fmt.key_size, 512)

        # no default for non-XTS modes
        fmt = LUKS(cipher="aes-cbc-plain64")
        self.assertEqual(fmt.key_size, 0)

    def test_luks2_pbkdf_memory_fips(self):
        fmt = LUKS(passphrase="passphrase")
        with patch("blivet.formats.luks.blockdev.crypto") as bd:
            # fips enabled, pbkdf memory should not be set
            with patch("blivet.formats.luks.crypto") as crypto:
                attrs = {"is_fips_enabled.return_value": True,
                         "get_optimal_luks_sector_size.return_value": 0,
                         "calculate_luks2_max_memory.return_value": Size("256 MiB")}
                crypto.configure_mock(**attrs)

                fmt._create()
                crypto.calculate_luks2_max_memory.assert_not_called()
                self.assertIsNone(bd.luks_format.call_args[1]["extra"])

            # fips disabled, pbkdf memory should be set
            with patch("blivet.formats.luks.crypto") as crypto:
                attrs = {"is_fips_enabled.return_value": False,
                         "get_optimal_luks_sector_size.return_value": 0,
                         "calculate_luks2_max_memory.return_value": Size("256 MiB")}
                crypto.configure_mock(**attrs)

                fmt._create()
                crypto.calculate_luks2_max_memory.assert_called()
                self.assertEqual(bd.luks_format.call_args[1]["extra"].pbkdf.max_memory_kb, 256 * 1024)

    def test_sector_size_luks1(self):
        fmt = LUKS(passphrase="passphrase")
        self.assertEqual(fmt.luks_sector_size, 0)

        # sector size is not valid for luks1
        with self.assertRaises(ValueError):
            fmt = LUKS(luks_version="luks1", luks_sector_size=4096, passphrase="passphrase")

        # just make sure we won't try to add the extra.sector_size argument ourselves
        fmt = LUKS(luks_version="luks1", passphrase="passphrase")
        with patch("blivet.devices.lvm.blockdev.crypto") as crypto:
            fmt._create()
            crypto.luks_format.assert_called()
            self.assertIsNone(crypto.luks_format.call_args[1]["extra"])

    def test_sector_size_luks2(self):
        fmt = LUKS(passphrase="passphrase")
        self.assertEqual(fmt.luks_sector_size, 0)

        fmt = LUKS(luks_version="luks2", luks_sector_size=4096, passphrase="passphrase")
        self.assertEqual(fmt.luks_sector_size, 4096)

        fmt = LUKS(passphrase="passphrase")
        with patch("blivet.devicelibs.crypto.calculate_luks2_max_memory", return_value=None):
            with patch("blivet.devicelibs.crypto.get_optimal_luks_sector_size", return_value=512):
                with patch("blivet.devices.lvm.blockdev.crypto") as crypto:
                    fmt._create()
                    crypto.luks_format.assert_called()
                    self.assertEqual(crypto.luks_format.call_args[1]["extra"].sector_size, 512)

        fmt = LUKS(passphrase="passphrase")
        with patch("blivet.devicelibs.crypto.calculate_luks2_max_memory", return_value=None):
            with patch("blivet.devicelibs.crypto.get_optimal_luks_sector_size", return_value=0):
                with patch("blivet.devices.lvm.blockdev.crypto") as crypto:
                    fmt._create()
                    crypto.luks_format.assert_called()
                    self.assertIsNone(crypto.luks_format.call_args[1]["extra"])



# Entry point for testing translatable strings
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Shea <dshea@redhat.com>

import sys
from . import testPOT

if len(sys.argv) < 2:
    print("Usage: translatable <POTfile>")
    sys.exit(1)

status = 0
for potfile in sys.argv[1:]:
    if not testPOT(potfile):
        status = 1

sys.exit(status)



#
# Copyright (C) 2016  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#
import dbus

from .constants import DEVICE_INTERFACE, DEVICE_OBJECT_PATH_BASE, DEVICE_REMOVED_OBJECT_PATH_BASE
from .object import DBusObject


class DBusDevice(DBusObject):
    def __init__(self, device, manager):
        self._device = device
        super(DBusDevice, self).__init__(manager)

    @property
    def id(self):
        return self._device.id

    @property
    def object_path(self):
        if self.present:
            base = DEVICE_OBJECT_PATH_BASE
        else:
            base = DEVICE_REMOVED_OBJECT_PATH_BASE

        return "%s/%s" % (base, self.id)

    @property
    def interface(self):
        return DEVICE_INTERFACE

    @property
    def properties(self):
        parents = (self._manager.get_object_by_id(d.id).object_path for d in self._device.parents)
        children = (self._manager.get_object_by_id(d.id).object_path for d in self._device.children)
        fmt = self._manager.get_object_by_id(self._device.format.id).object_path
        props = {"Name": self._device.name,
                 "Path": self._device.path,
                 "Type": self._device.type,
                 "Size": dbus.UInt64(self._device.size),
                 "ID": self._device.id,
                 "UUID": self._device.uuid or "",
                 "Status": self._device.status or False,
                 "RaidLevel": self._get_raid_level(),
                 "Parents": dbus.Array(parents, signature='o'),
                 "Children": dbus.Array(children, signature='o'),
                 "Format": dbus.ObjectPath(fmt)
                 }

        return props

    def _get_raid_level(self):
        level = ""
        if hasattr(self._device, "level"):
            level = str(self._device.level)
        elif hasattr(self._device, "data_level"):
            level = str(self._device.data_level)

        return level

    @dbus.service.method(dbus_interface=DEVICE_INTERFACE)
    def Setup(self):
        """ Activate this device. """
        self._device.setup()

    @dbus.service.method(dbus_interface=DEVICE_INTERFACE)
    def Teardown(self):
        """ Deactivate this device. """
        self._device.teardown()



try:
    from unittest.mock import patch, Mock, PropertyMock, sentinel
except ImportError:
    from mock import patch, Mock, PropertyMock, sentinel

import six
import unittest

from blivet.actionlist import ActionList
from blivet.errors import DeviceTreeError, DuplicateUUIDError, InvalidMultideviceSelection
from blivet.deviceaction import ACTION_TYPE_DESTROY, ACTION_OBJECT_DEVICE
from blivet.devicelibs import lvm
from blivet.devices import BTRFSSubVolumeDevice, BTRFSVolumeDevice
from blivet.devices import DiskDevice
from blivet.devices import LVMVolumeGroupDevice
from blivet.devices import LVMLogicalVolumeDevice
from blivet.devices import StorageDevice
from blivet.devices import MultipathDevice
from blivet.devices.lib import Tags
from blivet.devicetree import DeviceTree
from blivet.formats import get_format
from blivet.size import Size
from blivet.static_data.lvm_info import lvs_info, LVsInfo

"""
    TODO:

        - add more lvm tests
            - thin pool with separate data and metadata volumes?
            - raid lvs
            - raid thin pool
"""


class DeviceTreeTestCase(unittest.TestCase):

    def test_resolve_device(self):
        dt = DeviceTree()

        dev1_label = "dev1_label"
        dev1_uuid = "1234-56-7890"
        fmt1 = get_format("ext4", label=dev1_label, uuid=dev1_uuid)
        dev1 = StorageDevice("dev1", exists=True, fmt=fmt1, size=fmt1.min_size)
        dt._add_device(dev1)

        dev2_label = "dev2_label"
        fmt2 = get_format("swap", label=dev2_label)
        dev2 = StorageDevice("dev2", exists=True, fmt=fmt2)
        dt._add_device(dev2)

        dev3 = StorageDevice("sdp2", exists=True)
        dt._add_device(dev3)

        dev4 = StorageDevice("10", exists=True)
        dt._add_device(dev4)

        dt.edd_dict.update({"dev1": 0x81,
                            "dev2": 0x82})

        self.assertEqual(dt.resolve_device(dev1.name), dev1)
        self.assertEqual(dt.resolve_device("LABEL=%s" % dev1_label), dev1)
        self.assertEqual(dt.resolve_device("UUID=%s" % dev1_label), None)
        self.assertEqual(dt.resolve_device("UUID=%s" % dev1_uuid), dev1)
        self.assertEqual(dt.resolve_device("/dev/dev1"), dev1)

        self.assertEqual(dt.resolve_device("dev2"), dev2)
        self.assertEqual(dt.resolve_device("0x82"), dev2)

        self.assertEqual(dt.resolve_device(dev3.name), dev3)
        self.assertEqual(dt.resolve_device(dev4.name), dev4)

    def test_resolve_device_btrfs(self):
        dt = DeviceTree()

        # same uuid for both volume and subvolume
        btrfs_uuid = "1234-56-7890"

        dev = StorageDevice("deva", exists=True,
                            fmt=get_format("btrfs", uuid=btrfs_uuid, exists=True),
                            size=get_format("btrfs").min_size)
        dt._add_device(dev)

        vol = BTRFSVolumeDevice("vol", exists=True,
                                parents=[dev],
                                fmt=get_format("btrfs", uuid=btrfs_uuid, exists=True))
        dt._add_device(vol)

        sub1 = BTRFSSubVolumeDevice("sub1", exists=True,
                                    parents=[vol],
                                    fmt=get_format("btrfs", options="subvol=sub1",
                                                   uuid=btrfs_uuid, exists=True,
                                                   subvolspec="sub1"))
        dt._add_device(sub1)

        sub2 = BTRFSSubVolumeDevice("sub2", exists=True,
                                    parents=[vol],
                                    fmt=get_format("btrfs", options="subvol=sub2",
                                                   uuid=btrfs_uuid, exists=True,
                                                   subvolspec="sub2"))
        dt._add_device(sub2)

        # resolve with name should work as usual
        self.assertEqual(dt.resolve_device(vol.name), vol)
        self.assertEqual(dt.resolve_device(sub1.name), sub1)
        self.assertEqual(dt.resolve_device(sub2.name), sub2)

        # resolve by UUID with subvolspec
        self.assertEqual(dt.resolve_device("UUID=%s" % btrfs_uuid, subvolspec=sub1.format.subvolspec), sub1)
        self.assertEqual(dt.resolve_device("UUID=%s" % btrfs_uuid, subvolspec=sub2.format.subvolspec), sub2)

        # resolve by UUID with options
        self.assertEqual(dt.resolve_device("UUID=%s" % btrfs_uuid, options="subvol=%s" % sub1.name), sub1)
        self.assertEqual(dt.resolve_device("UUID=%s" % btrfs_uuid, options="subvol=%s" % sub2.name), sub2)

    def test_device_name(self):
        # check that devicetree.names property contains all device's names

        # mock lvs_info to avoid blockdev call allowing run as non-root
        with patch.object(LVsInfo, 'cache', new_callable=PropertyMock) as mock_lvs_cache:
            mock_lvs_cache.return_value = {"sdmock": "dummy", "testvg-testlv": "dummy"}

            tree = DeviceTree()
            dev_names = ["test_sda", "test_sdb", "test_sdc"]

            for dev_name in dev_names:
                dev = DiskDevice(dev_name, size=Size("1 GiB"))
                tree._add_device(dev)
                self.assertTrue(dev in tree.devices)
                self.assertTrue(dev.name in tree.names)

            dev.format = get_format("lvmpv", device=dev.path)
            vg = LVMVolumeGroupDevice("testvg", parents=[dev])
            tree._add_device(vg)
            dev_names.append(vg.name)

            lv = LVMLogicalVolumeDevice("testlv", parents=[vg])
            tree._add_device(lv)
            dev_names.append(lv.name)

            # frobnicate a bit with the hidden status of the devices:
            # * hide sda
            # * hide and unhide again sdb
            # * leave sdc unchanged
            tree.hide(tree.get_device_by_name("test_sda"))
            tree.hide(tree.get_device_by_name("test_sdb"))
            tree.unhide(tree.get_device_by_name("test_sdb", hidden=True))

            # some lvs names may be already present in the system (mocked)
            lv_info = list(lvs_info.cache.keys())

            # all devices should still be present in the tree.names
            self.assertEqual(set(tree.names), set(lv_info + dev_names))

            # "remove" the LV, it should no longer be in the list
            tree.actions._actions.append(Mock(device=lv, type=ACTION_TYPE_DESTROY,
                                              obj=ACTION_OBJECT_DEVICE))
            tree._remove_device(lv)
            self.assertFalse(lv.name in tree.names)

    # XXX: the lvm_devices_* functions are decorated with needs_config_refresh decorator which
    #      at this point is already applied as a no-op because LVM libblockdev plugin is not available
    @patch("blivet.devicelibs.lvm.lvm_devices_add", new=lvm._lvm_devices.add)
    @patch("blivet.devicelibs.lvm.lvm_devices_remove", new=lvm._lvm_devices.remove)
    @patch("blivet.devicelibs.lvm.lvm_devices_reset", new=lvm._lvm_devices.clear)
    def test_reset(self):
        dt = DeviceTree()
        names = ["fakedev1", "fakedev2"]
        for name in names:
            device = Mock(name=name, spec=StorageDevice, parents=[], exists=True)
            dt._devices.append(device)

        dt.actions._actions.append(Mock(name="fake action"))

        lvm.lvm_devices_add("xxx")

        dt.ignored_disks.append(names[0])
        dt.exclusive_disks.append(names[1])

        dt._hidden.append(dt._devices.pop(1))

        dt.edd_dict = {"a": 22}

        dt.reset()

        empty_list = list()
        self.assertEqual(dt._devices, empty_list)

        self.assertEqual(list(dt.actions), empty_list)
        self.assertIsInstance(dt.actions, ActionList)

        self.assertEqual(dt._hidden, empty_list)

        self.assertEqual(lvm._lvm_devices, set())

        self.assertEqual(dt.exclusive_disks, empty_list)
        self.assertEqual(dt.ignored_disks, empty_list)

        self.assertEqual(dt.edd_dict, dict())

    @patch.object(StorageDevice, "add_hook")
    @patch("blivet.static_data.lvm_info.blockdev.lvm.lvs", return_value=[])
    def test_add_device(self, *args):  # pylint: disable=unused-argument
        dt = DeviceTree()

        dev1 = StorageDevice("dev1", exists=False, uuid=sentinel.dev1_uuid, parents=[])

        self.assertEqual(dt.devices, list())

        # things are called, updated as expected when a device is added
        with patch("blivet.devicetree.callbacks") as callbacks:
            dt._add_device(dev1)
            self.assertTrue(callbacks.device_added.called)

        self.assertEqual(dt.devices, [dev1])
        self.assertTrue(dev1 in dt.devices)
        self.assertTrue(dev1.name in dt.names)
        self.assertTrue(dev1.add_hook.called)  # pylint: disable=no-member

        # adding an already-added device fails
        six.assertRaisesRegex(self, DeviceTreeError, "Trying to add already existing device.", dt._add_device, dev1)

        # adding a device with the same UUID
        dev_clone = StorageDevice("dev_clone", exists=False, uuid=sentinel.dev1_uuid, parents=[])
        six.assertRaisesRegex(self, DuplicateUUIDError, "Duplicate UUID.*", dt._add_device, dev_clone)

        dev2 = StorageDevice("dev2", exists=False, parents=[])
        dev3 = StorageDevice("dev3", exists=False, parents=[dev1, dev2])

        # adding a device with one or more parents not already in the tree fails
        six.assertRaisesRegex(self, DeviceTreeError, "parent.*not in tree", dt._add_device, dev3)
        self.assertFalse(dev2 in dt.devices)
        self.assertFalse(dev2.name in dt.names)

        dt._add_device(dev2)
        self.assertTrue(dev2 in dt.devices)
        self.assertTrue(dev2.name in dt.names)

        dt._add_device(dev3)
        self.assertTrue(dev3 in dt.devices)
        self.assertTrue(dev3.name in dt.names)

    @patch.object(StorageDevice, "remove_hook")
    @patch("blivet.static_data.lvm_info.blockdev.lvm.lvs", return_value=[])
    def test_remove_device(self, *args):  # pylint: disable=unused-argument
        dt = DeviceTree()

        dev1 = StorageDevice("dev1", exists=False, parents=[])

        # removing a device not in the tree raises an exception
        six.assertRaisesRegex(self, ValueError, "not in tree", dt._remove_device, dev1)

        dt._add_device(dev1)
        with patch("blivet.devicetree.callbacks") as callbacks:
            dt._remove_device(dev1)
            self.assertTrue(callbacks.device_removed.called)

        self.assertFalse(dev1 in dt.devices)
        self.assertFalse(dev1.name in dt.names)
        self.assertTrue(dev1.remove_hook.called)  # pylint: disable=no-member

        dev2 = StorageDevice("dev2", exists=False, parents=[dev1])
        dt._add_device(dev1)
        dt._add_device(dev2)
        self.assertTrue(dev2 in dt.devices)
        self.assertTrue(dev2.name in dt.names)

        # removal of a non-leaf device raises an exception
        six.assertRaisesRegex(self, ValueError, "non-leaf device", dt._remove_device, dev1)
        self.assertTrue(dev1 in dt.devices)
        self.assertTrue(dev1.name in dt.names)
        self.assertTrue(dev2 in dt.devices)
        self.assertTrue(dev2.name in dt.names)

        # forcing removal of non-leaf device does not remove the children
        dt._remove_device(dev1, force=True)
        self.assertFalse(dev1 in dt.devices)
        self.assertFalse(dev1.name in dt.names)
        self.assertTrue(dev2 in dt.devices)
        self.assertTrue(dev2.name in dt.names)

    def test_get_device_by_name(self):
        dt = DeviceTree()

        dev1 = StorageDevice("dev1", exists=False, parents=[])
        dev2 = StorageDevice("dev2", exists=False, parents=[dev1])
        dt._add_device(dev1)
        dt._add_device(dev2)

        self.assertIsNone(dt.get_device_by_name("dev3"))
        self.assertEqual(dt.get_device_by_name("dev2"), dev2)
        self.assertEqual(dt.get_device_by_name("dev1"), dev1)

        dev2.complete = False
        self.assertEqual(dt.get_device_by_name("dev2"), None)
        self.assertEqual(dt.get_device_by_name("dev2", incomplete=True), dev2)

        dev3 = StorageDevice("dev3", exists=True, parents=[])
        dt._add_device(dev3)
        dt.hide(dev3)
        self.assertIsNone(dt.get_device_by_name("dev3"))
        self.assertEqual(dt.get_device_by_name("dev3", hidden=True), dev3)

    def test_recursive_remove(self):
        dt = DeviceTree()
        dev1 = StorageDevice("dev1", exists=False, parents=[])
        dev2 = StorageDevice("dev2", exists=False, parents=[dev1])
        dt._add_device(dev1)
        dt._add_device(dev2)

        # normal
        self.assertTrue(dev1 in dt.devices)
        self.assertTrue(dev2 in dt.devices)
        self.assertEqual(dt.actions._actions, list())
        dt.recursive_remove(dev1)
        self.assertFalse(dev1 in dt.devices)
        self.assertFalse(dev2 in dt.devices)
        self.assertNotEqual(dt.actions._actions, list())

        dt.reset()
        dt._add_device(dev1)
        dt._add_device(dev2, new=False)  # restore parent/child relationships

        # remove_device clears descendants and formatting but preserves the device
        dev1.format = get_format("swap")
        self.assertEqual(dev1.format.type, "swap")
        self.assertEqual(dt.actions._actions, list())
        dt.recursive_remove(dev1, remove_device=False)
        self.assertTrue(dev1 in dt.devices)
        self.assertFalse(dev2 in dt.devices)
        self.assertEqual(dev1.format.type, None)
        self.assertNotEqual(dt.actions._actions, list())

        dt.reset()
        dt._add_device(dev1)
        dt._add_device(dev2, new=False)  # restore parent/child relationships

        # actions=False performs the removals without scheduling actions
        self.assertEqual(dt.actions._actions, list())
        dt.recursive_remove(dev1, actions=False)
        self.assertFalse(dev1 in dt.devices)
        self.assertFalse(dev2 in dt.devices)
        self.assertEqual(dt.actions._actions, list())

        dt.reset()
        dt._add_device(dev1)
        dt._add_device(dev2, new=False)  # restore parent/child relationships

        # modparent only works when actions=False is passed
        with patch.object(dt, "_remove_device") as remove_device:
            dt.recursive_remove(dev1, actions=False)
            remove_device.assert_called_with(dev1, modparent=True)

            dt.recursive_remove(dev1, actions=False, modparent=False)
            remove_device.assert_called_with(dev1, modparent=False)

    def test_ignored_disk_tags(self):
        tree = DeviceTree()

        fake_ssd = Mock(name="fake_ssd", spec=StorageDevice, parents=[],
                        tags=[Tags.ssd], exists=True)
        fake_local = Mock(name="fake_local", spec=StorageDevice, parents=[],
                          tags=[Tags.local], exists=True)
        tree._devices.extend([fake_ssd, fake_local])

        self.assertFalse(tree._is_ignored_disk(fake_ssd))
        self.assertFalse(tree._is_ignored_disk(fake_local))
        tree.ignored_disks.append("@ssd")
        self.assertTrue(tree._is_ignored_disk(fake_ssd))
        self.assertFalse(tree._is_ignored_disk(fake_local))
        tree.exclusive_disks.append("@local")
        self.assertTrue(tree._is_ignored_disk(fake_ssd))
        self.assertFalse(tree._is_ignored_disk(fake_local))

    def test_expand_taglist(self):
        tree = DeviceTree()

        sda = DiskDevice("test_sda")
        sdb = DiskDevice("test_sdb")
        sdc = DiskDevice("test_sdc")
        sdd = DiskDevice("test_sdd")

        tree._add_device(sda)
        tree._add_device(sdb)
        tree._add_device(sdc)
        tree._add_device(sdd)

        sda.tags = {Tags.remote}
        sdb.tags = {Tags.ssd}
        sdc.tags = {Tags.local, Tags.ssd}
        sdd.tags = set()

        self.assertEqual(tree.expand_taglist(["test_sda", "test_sdb"]), {"test_sda", "test_sdb"})
        self.assertEqual(tree.expand_taglist(["@local"]), {"test_sdc"})
        self.assertEqual(tree.expand_taglist(["@ssd"]), {"test_sdb", "test_sdc"})
        self.assertEqual(tree.expand_taglist(["@ssd", "test_sdd", "@local"]), {"test_sdb", "test_sdc", "test_sdd"})
        with self.assertRaises(ValueError):
            tree.expand_taglist(["test_sdd", "@invalid_tag"])

    def test_hide_ignored_disks(self):
        tree = DeviceTree()

        sda = DiskDevice("test_sda")
        sdb = DiskDevice("test_sdb")
        sdc = DiskDevice("test_sdc")

        tree._add_device(sda)
        tree._add_device(sdb)
        tree._add_device(sdc)

        self.assertTrue(sda in tree.devices)
        self.assertTrue(sdb in tree.devices)
        self.assertTrue(sdc in tree.devices)

        # test ignored_disks
        tree.ignored_disks = ["test_sdb"]

        # verify hide is called as expected
        with patch.object(tree, "hide") as hide:
            tree._hide_ignored_disks()
            hide.assert_called_with(sdb)

        # verify that hide works as expected
        tree._hide_ignored_disks()
        self.assertTrue(sda in tree.devices)
        self.assertFalse(sdb in tree.devices)
        self.assertTrue(sdc in tree.devices)

        # unhide sdb and make sure it works
        tree.unhide(sdb)
        self.assertTrue(sda in tree.devices)
        self.assertTrue(sdb in tree.devices)
        self.assertTrue(sdc in tree.devices)

        # test exclusive_disks
        tree.ignored_disks = []
        tree.exclusive_disks = ["test_sdc"]
        with patch.object(tree, "hide") as hide:
            tree._hide_ignored_disks()
            hide.assert_any_call(sda)
            hide.assert_any_call(sdb)

        tree._hide_ignored_disks()
        self.assertFalse(sda in tree.devices)
        self.assertFalse(sdb in tree.devices)
        self.assertTrue(sdc in tree.devices)

        tree.unhide(sda)
        tree.unhide(sdb)
        self.assertTrue(sda in tree.devices)
        self.assertTrue(sdb in tree.devices)
        self.assertTrue(sdc in tree.devices)

    def test_get_related_disks(self):
        tree = DeviceTree()

        sda = DiskDevice("test_sda", size=Size('300g'), exists=False)
        sdb = DiskDevice("test_sdb", size=Size('300g'), exists=False)
        sdc = DiskDevice("test_sdc", size=Size('300G'), exists=False)

        tree._add_device(sda)
        tree._add_device(sdb)
        tree._add_device(sdc)

        self.assertTrue(sda in tree.devices)
        self.assertTrue(sdb in tree.devices)
        self.assertTrue(sdc in tree.devices)

        sda.format = get_format("lvmpv", device=sda.path)
        sdb.format = get_format("lvmpv", device=sdb.path)
        vg = LVMVolumeGroupDevice("relvg", parents=[sda, sdb])
        tree._add_device(vg)

        self.assertEqual(tree.get_related_disks(sda), set([sda, sdb]))
        self.assertEqual(tree.get_related_disks(sdb), set([sda, sdb]))
        self.assertEqual(tree.get_related_disks(sdc), set())
        tree.hide(sda)
        self.assertEqual(tree.get_related_disks(sda), set([sda, sdb]))
        self.assertEqual(tree.get_related_disks(sdb), set([sda, sdb]))
        tree.hide(sdb)
        self.assertEqual(tree.get_related_disks(sda), set([sda, sdb]))
        self.assertEqual(tree.get_related_disks(sdb), set([sda, sdb]))
        tree.unhide(sda)
        self.assertEqual(tree.get_related_disks(sda), set([sda, sdb]))
        self.assertEqual(tree.get_related_disks(sdb), set([sda, sdb]))

    # XXX: the lvm_devices_* functions are decorated with needs_config_refresh decorator which
    #      at this point is already applied as a no-op because LVM libblockdev plugin is not available
    @patch("blivet.devicelibs.lvm.lvm_devices_add", new=lvm._lvm_devices.add)
    @patch("blivet.devicelibs.lvm.lvm_devices_remove", new=lvm._lvm_devices.remove)
    def test_lvm_filter_hide_unhide(self):
        tree = DeviceTree()

        sda = DiskDevice("test_sda", size=Size("30 GiB"))
        sdb = DiskDevice("test_sdb", size=Size("30 GiB"))

        tree._add_device(sda)
        tree._add_device(sdb)

        self.assertTrue(sda in tree.devices)
        self.assertTrue(sdb in tree.devices)

        sda.format = get_format("lvmpv", device=sda.path)
        sdb.format = get_format("lvmpv", device=sdb.path)

        # LVMPhysicalVolume._create would do this
        lvm.lvm_devices_add(sda.path)
        lvm.lvm_devices_add(sdb.path)

        self.assertSetEqual(lvm._lvm_devices, {sda.path, sdb.path})

        tree.hide(sda)
        self.assertSetEqual(lvm._lvm_devices, {sdb.path})
        tree.hide(sdb)
        self.assertSetEqual(lvm._lvm_devices, set())

        tree.unhide(sda)
        self.assertSetEqual(lvm._lvm_devices, {sda.path})
        tree.unhide(sdb)
        self.assertSetEqual(lvm._lvm_devices, {sda.path, sdb.path})


class DeviceTreeIgnoredExclusiveMultipathTestCase(unittest.TestCase):

    def setUp(self):
        self.tree = DeviceTree()

        self.sda = DiskDevice("test_sda")
        self.sdb = DiskDevice("test_sdb")
        self.sdc = DiskDevice("test_sdc")

        self.tree._add_device(self.sda)
        self.tree._add_device(self.sdb)
        self.tree._add_device(self.sdc)

        self.assertTrue(self.sda in self.tree.devices)
        self.assertTrue(self.sdb in self.tree.devices)
        self.assertTrue(self.sdc in self.tree.devices)

        # now test exclusive_disks special cases for multipath
        self.sda.format = get_format("multipath_member", exists=True)
        self.sdb.format = get_format("multipath_member", exists=True)
        self.sdc.format = get_format("multipath_member", exists=True)
        self.mpatha = MultipathDevice("mpatha", parents=[self.sda, self.sdb, self.sdc])
        self.tree._add_device(self.mpatha)

    def test_exclusive_disks_multipath_1(self):
        # multipath is exclusive -> all disks should be exclusive
        self.tree.ignored_disks = []
        self.tree.exclusive_disks = ["mpatha"]

        with patch.object(self.tree, "hide") as hide:
            self.tree._hide_ignored_disks()
            self.assertFalse(hide.called)

        self.tree._hide_ignored_disks()
        self.assertTrue(self.sda in self.tree.devices)
        self.assertTrue(self.sdb in self.tree.devices)
        self.assertTrue(self.sdc in self.tree.devices)
        self.assertTrue(self.mpatha in self.tree.devices)

    def test_exclusive_disks_multipath_2(self):
        # all disks exclusive -> mpath should also be exclusive
        self.tree.exclusive_disks = ["test_sda", "test_sdb", "test_sdc"]
        with patch.object(self.tree, "hide") as hide:
            self.tree._hide_ignored_disks()
            self.assertFalse(hide.called)

        self.tree._hide_ignored_disks()
        self.assertTrue(self.sda in self.tree.devices)
        self.assertTrue(self.sdb in self.tree.devices)
        self.assertTrue(self.sdc in self.tree.devices)
        self.assertTrue(self.mpatha in self.tree.devices)

    def test_exclusive_disks_multipath_3(self):
        # some disks exclusive -> mpath should be hidden
        self.tree.exclusive_disks = ["test_sda", "test_sdb"]
        with patch.object(self.tree, "hide") as hide:
            self.tree._hide_ignored_disks()
            hide.assert_any_call(self.mpatha)
            hide.assert_any_call(self.sdc)

        # verify that hide works as expected
        self.tree._hide_ignored_disks()
        self.assertTrue(self.sda in self.tree.devices)
        self.assertTrue(self.sdb in self.tree.devices)
        self.assertFalse(self.sdc in self.tree.devices)
        self.assertFalse(self.mpatha in self.tree.devices)

    def test_ignored_disks_multipath_1(self):
        # mpatha ignored -> disks should be hidden
        self.tree.ignored_disks = ["mpatha"]
        self.tree.exclusive_disks = []

        with patch.object(self.tree, "hide") as hide:
            self.tree._hide_ignored_disks()
            hide.assert_any_call(self.mpatha)
            hide.assert_any_call(self.sda)
            hide.assert_any_call(self.sdb)
            hide.assert_any_call(self.sdc)

        self.tree._hide_ignored_disks()
        self.assertFalse(self.sda in self.tree.devices)
        self.assertFalse(self.sdb in self.tree.devices)
        self.assertFalse(self.sdc in self.tree.devices)
        self.assertFalse(self.mpatha in self.tree.devices)

    def test_ignored_disks_multipath_2(self):
        # all disks ignored -> mpath should be hidden
        self.tree.ignored_disks = ["test_sda", "test_sdb", "test_sdc"]
        self.tree.exclusive_disks = []

        with patch.object(self.tree, "hide") as hide:
            self.tree._hide_ignored_disks()
            hide.assert_any_call(self.mpatha)
            hide.assert_any_call(self.sda)
            hide.assert_any_call(self.sdb)
            hide.assert_any_call(self.sdc)

        self.tree._hide_ignored_disks()
        self.assertFalse(self.sda in self.tree.devices)
        self.assertFalse(self.sdb in self.tree.devices)
        self.assertFalse(self.sdc in self.tree.devices)
        self.assertFalse(self.mpatha in self.tree.devices)

    def test_ignored_disks_multipath_3(self):
        # some disks ignored -> error
        self.tree.ignored_disks = ["test_sda", "test_sdb"]
        self.tree.exclusive_disks = []

        with self.assertRaises(InvalidMultideviceSelection):
            self.tree._hide_ignored_disks()



# Check what percentage of strings a .po translates
#
# This will reject translations that fall below a certain threshold of
# translated strings.
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Shea <dshea@redhat.com>

import warnings

try:
    import polib
except ImportError:
    print("You need to install the python-polib package to read translations")
    raise

threshold = 10

def test_percentage(pofile):
    pofile = polib.pofile(pofile)
    if pofile.percent_translated() < threshold:
        # Issue a warning instead of an exception, since these should probably
        # be handled on a case-by-case basis
        warnings.warn("amount translated of %d%% below threshold of %d%%" % (pofile.percent_translated(), threshold))



# Check that a string does not contain unnecessary Pango markup
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Shea <dshea@redhat.com>

from pocketlint.pangocheck import is_markup, markup_necessary
import xml.etree.ElementTree as ET

def test_markup(poentry):
    # Unnecessary markup is markup applied to an entire string, such as
    # _("<b>Bold Text</b>"). This could be instead be translated as
    # "<b>%s</b>" % _("Bold Text"), and then the translator doesn't have to see
    # the markup at all.

    if is_markup(poentry.msgid):
        # Wrap the string in <markup> nodes, parse it, test it
        # The markup is unescaped on purpose
        # pylint: disable=unescaped-markup
        tree = ET.fromstring("<markup>%s</markup>" % poentry.msgid)
        if not markup_necessary(tree):
            raise AssertionError("Unnecessary markup")



# fsreadlabel.py
# Filesystem label reading classes.
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Anne Mulhern <amulhern@redhat.com>

import abc
import re

from six import add_metaclass

from ..errors import FSReadLabelError
from .. import util

from . import availability
from . import fstask
from . import task


@add_metaclass(abc.ABCMeta)
class FSReadLabel(task.BasicApplication, fstask.FSTask):

    """ An abstract class that represents reading a filesystem's label. """
    description = "read filesystem label"

    label_regex = abc.abstractproperty(
        doc="Matches the string output by the reading application.")

    args = abc.abstractproperty(doc="arguments for reading a label.")

    # IMPLEMENTATION methods

    @property
    def _read_command(self):
        """Get the command to read the filesystem label.

           :return: the command
           :rtype: list of str
        """
        return [str(self.ext)] + self.args

    def _extract_label(self, labelstr):
        """Extract the label from an output string.

           :param str labelstr: the string containing the label information

           :return: the label
           :rtype: str

           Raises an FSReadLabelError if the label can not be extracted.
        """
        match = re.match(self.label_regex, labelstr)
        if match is None:
            raise FSReadLabelError("Unknown format for application %s" % self.ext)
        return match.group('label')

    def do_task(self):  # pylint: disable=arguments-differ
        """ Get the label.

            :returns: the filesystem label
            :rtype: str
        """
        error_msgs = self.availability_errors
        if error_msgs:
            raise FSReadLabelError("\n".join(error_msgs))

        (rc, out) = util.run_program_and_capture_output(self._read_command)
        if rc != 0:
            raise FSReadLabelError("read label failed")

        label = out.strip()

        return label if label == "" else self._extract_label(label)


class DosFSReadLabel(FSReadLabel):
    ext = availability.DOSFSLABEL_APP
    label_regex = r'(?P<label>.*)'

    @property
    def args(self):
        return [self.fs.device]


class Ext2FSReadLabel(FSReadLabel):
    ext = availability.E2LABEL_APP
    label_regex = r'(?P<label>.*)'

    @property
    def args(self):
        return [self.fs.device]


class NTFSReadLabel(FSReadLabel):
    ext = availability.NTFSLABEL_APP
    label_regex = r'(?P<label>.*)'

    @property
    def args(self):
        return [self.fs.device]


class XFSReadLabel(FSReadLabel):
    ext = availability.XFSADMIN_APP
    label_regex = r'label = "(?P<label>.*)"'

    @property
    def args(self):
        return ["-l", self.fs.device]


class UnimplementedFSReadLabel(fstask.UnimplementedFSTask):
    pass



# vim:set fileencoding=utf-8
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

import six
import unittest

from blivet.devices import LVMVolumeGroupDevice
from blivet.devices import LVMLogicalVolumeDevice
from blivet.devices import StorageDevice
from blivet.size import Size
import blivet


class DeviceNameTestCase(unittest.TestCase):
    """Test device name validation"""
    @patch.object(StorageDevice, "status", return_value=True)
    @patch.object(StorageDevice, "update_sysfs_path", return_value=None)
    @patch.object(StorageDevice, "read_current_size", return_value=None)
    def test_storage_device(self, *patches):  # pylint: disable=unused-argument
        # Check that / and NUL are rejected along with . and ..
        good_names = ['sda1', '1sda', 'good-name', 'cciss/c0d0']
        bad_names = ['sda/1', 'sda\x00', '.', '..', 'cciss/..']

        sd = StorageDevice("tester")

        for name in good_names:
            self.assertTrue(sd.is_name_valid(name))

        for name in bad_names:
            self.assertFalse(sd.is_name_valid(name))

        # Check that name validity check is omitted (only) when
        # device already exists
        # This test was added to prevent regression (see #1379145)
        for name in good_names:
            try:
                StorageDevice(name, exists=True)
            except ValueError:
                self.fail("Name check should not be performed nor failing")

            try:
                StorageDevice(name, exists=False)
            except ValueError:
                self.fail("Device name check failed when it shouldn't")

        for name in bad_names:
            try:
                StorageDevice(name, exists=True)
            except ValueError as e:
                if ' is not a valid name for this device' in str(e):
                    self.fail("Device name checked on already existing device")

            with six.assertRaisesRegex(self, ValueError, ' is not a valid name for this device'):
                StorageDevice(name, exists=False)

    def test_volume_group(self):
        good_names = ['vg00', 'group-name', 'groupname-']
        bad_names = ['-leading-hyphen', 'únicode', 'sp aces']

        pv = StorageDevice("pv1", fmt=blivet.formats.get_format("lvmpv"),
                           size=Size("1 GiB"))
        vg = LVMVolumeGroupDevice("testvg", parents=[pv])

        for name in good_names:
            self.assertTrue(vg.is_name_valid(name))

        for name in bad_names:
            self.assertFalse(vg.is_name_valid(name))

    def test_logical_volume(self):
        good_names = ['lv00', 'volume-name', 'volumename-']
        bad_names = ['-leading-hyphen', 'únicode', 'sp aces',
                     'snapshot47', 'pvmove0', 'sub_tmetastring']

        pv = StorageDevice("pv1", fmt=blivet.formats.get_format("lvmpv"),
                           size=Size("1 GiB"))
        vg = LVMVolumeGroupDevice("testvg", parents=[pv])
        lv = LVMLogicalVolumeDevice("testlv", parents=[vg],
                                    fmt=blivet.formats.get_format("xfs"))

        for name in good_names:
            self.assertTrue(lv.is_name_valid(name))

        for name in bad_names:
            self.assertFalse(lv.is_name_valid(name))



# fsck.py
# Filesystem check functionality.
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Anne Mulhern <amulhern@redhat.com>

import abc

from six import add_metaclass

from ..errors import FSError
from .. import util

from . import availability
from . import fstask
from . import task

_UNKNOWN_RC_MSG = "Unknown return code: %d"


@add_metaclass(abc.ABCMeta)
class FSCK(task.BasicApplication, fstask.FSTask):

    """An abstract class that represents actions associated with
       checking consistency of a filesystem.
    """
    description = "fsck"

    options = abc.abstractproperty(
        doc="Options for invoking the application.")

    # IMPLEMENTATION methods

    @abc.abstractmethod
    def _error_message(self, rc):
        """ Error message corresponding to rc.

            :param int rc: the fsck program return code
            :returns: an error message corresponding to the code, or None
            :rtype: str or NoneType

            A return value of None indicates no error.
        """
        raise NotImplementedError()

    @property
    def _fsck_command(self):
        """The command to check the filesystem.

           :return: the command
           :rtype: list of str
        """
        return [str(self.ext)] + self.options + [self.fs.device]

    def do_task(self):  # pylint: disable=arguments-differ
        """ Check the filesystem.

           :raises FSError: on failure
        """
        error_msgs = self.availability_errors
        if error_msgs:
            raise FSError("\n".join(error_msgs))

        try:
            rc = util.run_program(self._fsck_command)
        except OSError as e:
            raise FSError("filesystem check failed: %s" % e)

        error_msg = self._error_message(rc)
        if error_msg is not None:
            hdr = "%(type)s filesystem check failure on %(device)s: " % \
                {"type": self.fs.type, "device": self.fs.device}

            raise FSError(hdr + error_msg)


class DosFSCK(FSCK):
    _fsck_errors = {1: "Recoverable errors have been detected or dosfsck has "
                    "discovered an internal inconsistency.",
                    2: "Usage error."}

    ext = availability.DOSFSCK_APP
    options = ["-n"]

    def _error_message(self, rc):
        if rc < 1:
            return None
        try:
            return self._fsck_errors[rc]
        except KeyError:
            return _UNKNOWN_RC_MSG % rc


class Ext2FSCK(FSCK):
    _fsck_errors = {4: "File system errors left uncorrected.",
                    8: "Operational error.",
                    16: "Usage or syntax error.",
                    32: "e2fsck cancelled by user request.",
                    128: "Shared library error."}

    ext = availability.E2FSCK_APP
    # "Force checking even if the file system seems clean." (we might get false results otherwise)
    # + "Automatically repair ("preen") the file system."
    options = ["-f", "-p"]

    def _error_message(self, rc):
        msgs = (self._fsck_errors[c] for c in self._fsck_errors.keys() if rc & c)
        return "\n".join(msgs) or None


class XFSCK(FSCK):
    _fsck_errors = {1: "Runtime error encountered during repair operation.",
                    2: "XFS repair was unable to proceed due to a dirty log."}

    ext = availability.XFSREPAIR_APP
    options = []

    def _error_message(self, rc):
        msgs = (self._fsck_errors[c] for c in self._fsck_errors.keys() if rc & c)
        return "\n".join(msgs) or None


class HFSPlusFSCK(FSCK):
    _fsck_errors = {3: "Quick check found a dirty filesystem; no repairs done.",
                    4: "Root filesystem was dirty. System should be rebooted.",
                    8: "Corrupt filesystem, repairs did not succeed.",
                    47: "Major error found; no repairs attempted."}
    ext = availability.FSCK_HFSPLUS_APP
    options = []

    def _error_message(self, rc):
        if rc < 1:
            return None
        try:
            return self._fsck_errors[rc]
        except KeyError:
            return _UNKNOWN_RC_MSG % rc


class NTFSFSCK(FSCK):
    ext = availability.NTFSRESIZE_APP
    options = ["-c"]

    def _error_message(self, rc):
        return _UNKNOWN_RC_MSG % (rc,) if rc != 0 else None


class F2FSFSCK(FSCK):
    ext = availability.FSCK_F2FS_APP
    options = ["-a"]

    def _error_message(self, rc):
        return _UNKNOWN_RC_MSG % (rc,) if rc != 0 else None


class UnimplementedFSCK(fstask.UnimplementedFSTask):
    pass



#
# Copyright (C) 2016  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#
import dbus

from .constants import ACTION_INTERFACE, ACTION_OBJECT_PATH_BASE
from .object import DBusObject


class DBusAction(DBusObject):
    def __init__(self, action, manager):
        self._action = action
        super(DBusAction, self).__init__(manager)

    @property
    def id(self):
        return self._action.id

    @property
    def object_path(self):
        return "%s/%d" % (ACTION_OBJECT_PATH_BASE, self.id)

    @property
    def interface(self):
        return ACTION_INTERFACE

    @property
    def properties(self):
        props = {"Description": str(self._action),
                 "Device": dbus.ObjectPath(self._manager.get_object_by_id(self._action.device.id).object_path),
                 "Format": dbus.ObjectPath(self._manager.get_object_by_id(self._action.format.id).object_path),
                 "Type": "%s %s" % (self._action.type_string, self._action.object_string),
                 "ID": self._action.id}
        return props



# Framework for testing translations
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Shea <dshea@redhat.com>

"""
Framework for running tests against translations.

Tests are loaded from modules in this directory. A test is any callable object
within the module with a name that starts with 'test_'.

Each test is called with the name of .po file to test as an argument. A test
passes if it returns without raising an exception.
"""

import os, warnings

_tests = []

# Gather tests from this directory
import pkgutil
import importlib

for finder, mod_name, _ispkg in pkgutil.iter_modules(__path__):
    # Skip __main__
    if mod_name == "__main__":
        continue

    # Load the module
    full_name = "{}.{}".format(__name__, mod_name)
    module = importlib.import_module(full_name)

    # Look for attributes that start with 'test_' and add them to the test list
    for attrname, attr in module.__dict__.items():
        if attrname.startswith('test_') and callable(attr):
            _tests.append(attr)

def _remove_lingua(linguas, language):
    # Read in the LINGUAS file
    with open(linguas, "rt") as f:
        lingua_lines = f.readlines()

    output_lines = []
    for line in lingua_lines:
        # Leave comments alone
        if line.startswith('#'):
            output_lines.append(line)
            continue

        # Split the line into a list of languages, remove the one we don't
        # want, and put it back together
        lingua_list = line.split()
        lingua_list.remove(language)
        output_lines.append(" ".join(lingua_list))

    # Write LINGUAS back out
    with open(linguas, "wt") as f:
        f.writelines(output_lines)

def testFile(pofile, prefix=None, releaseMode=False, modifyLinguas=True):
    """Run all registered tests against the given .mo file.

       If run in release mode, this function will always return true, and if
       the mofile does not pass the tests the langauge will be removed.

       :param str mofile: The .mo file name to check
       :param str prefix: An optional directory prefix to strip from error messages
       :param bool releaseMode: whether to run in release mode
       :param bool modifyLinguas: whether to remove translations from LINGUAS in release mode
       :return: whether the checks succeeded or not
       :rtype: bool
    """
    success = True
    for test in _tests:
        # Don't print the tmpdir path in error messages
        if prefix is not None and pofile.startswith(prefix):
            poerror = pofile[len(prefix):]
        else:
            poerror = pofile

        try:
            with warnings.catch_warnings(record=True) as w:
                test(pofile)

                # Print any warnings collected
                for warn in w:
                    print("%s warned on %s: %s" % (test.__name__, poerror, warn.message))
        except Exception as e: # pylint: disable=broad-except
            print("%s failed on %s: %s" % (test.__name__, poerror, str(e)))
            if releaseMode:
                # Remove the po file and the .mo file built from it
                print("Removing %s" % pofile)
                os.remove(pofile)

                # Check for both .mo and .gmo
                mofile = os.path.splitext(pofile)[0] + '.mo'
                if os.path.exists(mofile):
                    print("Removing %s" % mofile)
                    os.remove(mofile)

                gmofile = os.path.splitext(pofile)[0] + '.gmo'
                if os.path.exists(gmofile):
                    print("Removing %s" % gmofile)
                    os.remove(gmofile)

                if modifyLinguas:
                    # If there is a LINGUAS file in the po directory, remove the
                    # language from it
                    linguas = os.path.join(os.path.dirname(mofile), 'LINGUAS')
                    if os.path.exists(linguas):
                        language = os.path.splitext(os.path.basename(pofile))[0]
                        print("Removing %s from LINGUAS" % language)
                        _remove_lingua(linguas, language)

                # No need to run the rest of the tests since we just killed the file
                break
            else:
                success = False

    return success

def testSourceTree(srcdir, releaseMode=False, modifyLinguas=True):
    """Runs all registered tests against all .po files in the given directory.

       If run in release mode, this function will always return True and the
       languages that do not pass the tests will be removed.

       :param str srcdir: The path to the source directory to check
       :param bool releaseMode: whether to run in release mode
       :param bool modifyLinguas: whether to remove translations from LINGUAS in release mode
       :return: whether the checks succeeded or not
       :rtype: bool
    """
    success = True
    srcdir = os.path.normpath(srcdir)

    for dirpath, _dirnames, paths in os.walk(srcdir):
        for pofile in (os.path.join(dirpath, path) for path in paths if path.endswith('.po')):
            if not testFile(pofile, prefix=srcdir + "/", releaseMode=releaseMode, modifyLinguas=modifyLinguas):
                success = False

    return success
